//! @file objectmanager.cpp
//! @brief ObjectManagerNX̒`

//--------------------------------------------------------------------------------
// 
// OpenXOPS
// Copyright (c) 2014-2022, OpenXOPS Project / [-_-;](mikan) All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice, 
//   this list of conditions and the following disclaimer in the documentation 
//   and/or other materials provided with the distribution.
// * Neither the name of the OpenXOPS Project nor the names of its contributors 
//   may be used to endorse or promote products derived from this software 
//   without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL OpenXOPS Project BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//--------------------------------------------------------------------------------

#include "objectmanager.h"

//! @brief RXgN^
ObjectManager::ObjectManager()
{
	HumanIndex = new human[MAX_HUMAN];
	WeaponIndex = new weapon[MAX_WEAPON];
	SmallObjectIndex = new smallobject[MAX_SMALLOBJECT];
	BulletIndex = new bullet[MAX_BULLET];
	GrenadeIndex = new grenade[MAX_GRENADE];
	EffectIndex = new effect[MAX_EFFECT];

	framecnt = 0;
	Human_ontarget = new float[MAX_HUMAN];
	Human_kill = new int[MAX_HUMAN];
	Human_headshot = new int[MAX_HUMAN];
	Human_ShotFlag = new bool[MAX_HUMAN];
	BulletObj_HumanIndex = new BulletObjectHumanIndex[MAX_BULLET];
	AddCollisionFlag = false;
	FriendlyFire = new bool[MAX_HUMAN];
	Player_HumanID = 0;
	Human_FrameTextureRefresh = new bool[MAX_HUMAN];

	GameParamInfo = NULL;
	d3dg = NULL;
	Resource = NULL;
	BlockData = NULL;
	PointData = NULL;
	CollD = NULL;
	GameSound = NULL;
	MIFdata = NULL;

	ObjectLog = new ObjectManagerLog;
}

//! @brief fBXgN^
ObjectManager::~ObjectManager()
{
	Cleanup();

	if( HumanIndex != NULL ){ delete [] HumanIndex; }
	if( WeaponIndex != NULL ){ delete [] WeaponIndex; }
	if( SmallObjectIndex != NULL ){ delete [] SmallObjectIndex; }
	if( BulletIndex != NULL ){ delete [] BulletIndex; }
	if( GrenadeIndex != NULL ){ delete [] GrenadeIndex; }
	if( EffectIndex != NULL ){ delete [] EffectIndex; }

	if( Human_ontarget != NULL ){ delete [] Human_ontarget; }
	if( Human_kill != NULL ){ delete [] Human_kill; }
	if( Human_headshot != NULL ){ delete [] Human_headshot; }
	if( Human_ShotFlag != NULL ){ delete [] Human_ShotFlag; }
	if( BulletObj_HumanIndex != NULL ){ delete [] BulletObj_HumanIndex; }
	if( FriendlyFire != NULL ){ delete [] FriendlyFire; }
	if( Human_FrameTextureRefresh != NULL ){ delete [] Human_FrameTextureRefresh; }

	if( ObjectLog != NULL ){ delete ObjectLog; }
}

//! @brief QƂNXݒ
//! @param in_GameParamInfo Q[ݒf[^ǗNX
//! @param in_d3dg `揈NX
//! @param in_Resource \[XǗNX
//! @param in_BlockData ubNf[^ǗNX
//! @param in_PointData |Cgf[^ǗNX
//! @param in_CollD 蔻菈NX
//! @param in_GameSound ʉĐNX
//! @param in_MIFdata MIFRg[NX
//! @attention ̊֐ŐݒsȂƁANX̂@\܂B
void ObjectManager::SetClass(ParameterInfo *in_GameParamInfo, D3DGraphics *in_d3dg, ResourceManager *in_Resource, BlockDataInterface *in_BlockData, PointDataInterface *in_PointData, Collision *in_CollD, SoundManager *in_GameSound, MIFInterface *in_MIFdata)
{
	GameParamInfo = in_GameParamInfo;
	d3dg = in_d3dg;
	Resource = in_Resource;
	BlockData = in_BlockData;
	PointData = in_PointData;
	CollD = in_CollD;
	GameSound = in_GameSound;
	MIFdata = in_MIFdata;

	for(int i=0; i<MAX_HUMAN; i++){
		HumanIndex[i].SetParameterInfoClass(GameParamInfo);
	}
	for(int i=0; i<MAX_WEAPON; i++){
		WeaponIndex[i].SetParameterInfoClass(GameParamInfo);
	}
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		SmallObjectIndex[i].SetParameterInfoClass(GameParamInfo);
		SmallObjectIndex[i].SetMIFInterfaceClass(MIFdata);
	}
	for(int i=0; i<MAX_GRENADE; i++){
		GrenadeIndex[i].SetParameterInfoClass(GameParamInfo);
	}

	int bulletmodel, bullettexture;
	BulletParameter data;

	if( Resource->GetBulletModelTexture(0, &bulletmodel, &bullettexture) == 0 ){
		if( GameParamInfo->GetBullet(0, &data) == 0 ){
			for(int i=0; i<MAX_BULLET; i++){
				BulletIndex[i].SetModel(bulletmodel, data.size);
				BulletIndex[i].SetTexture(bullettexture);
			}
		}
	}

	if( Resource->GetBulletModelTexture(1, &bulletmodel, &bullettexture) == 0 ){
		if( GameParamInfo->GetBullet(1, &data) == 0 ){
			for(int i=0; i<MAX_GRENADE; i++){
				GrenadeIndex[i].SetModel(bulletmodel, data.size);
				GrenadeIndex[i].SetTexture(bullettexture);
			}
		}
	}

	ObjectLog->SetClass(d3dg);
}

//! @brief lǉ
//! @param data l̃|Cgf[^@ipointdata\́j
//! @param infodata QƂl̃|Cgf[^@iVj
//! @param RandomFlag _␳Lɂ
//! @return Ff[^ԍi0ȏj@sF-1
//! @attention Ȑl̎ޔԍw肳ꂽꍇ ʏ́Fl oꂵ܂BeNX`̓}bveNX`0ԂgpAHP 0 wij܂B
int ObjectManager::AddHumanIndex(pointdata data, pointdata infodata, bool RandomFlag)
{
	int GetHumanFlag;
	HumanParameter HumanParam;
	GetHumanFlag = GameParamInfo->GetHuman(infodata.p2, &HumanParam);
	int Humanindexid = -1;
	int Weaponindexid = -1;
	class weapon *Weapon[TOTAL_HAVEWEAPON];
	int lnbslist[TOTAL_HAVEWEAPON];
	int nbslist[TOTAL_HAVEWEAPON];
	for(int i=0; i<TOTAL_HAVEWEAPON; i++){
		Weapon[i] = NULL;
		lnbslist[i] = 0;
		nbslist[i] = 0;
	}

#ifndef ENABLE_BUG_HUMANWEAPON
	if( GetHumanFlag != 0 ){
		return -1;
	}
#endif

	//l̃fԍ擾
	int upmodel[TOTAL_UPMODE];
	int armmodel[TOTAL_ARMMODE];
	int legmodel;
	int walkmodel[TOTAL_WALKMODE];
	int runmodel[TOTAL_RUNMODE];
	Resource->GetHumanModel(upmodel, armmodel, &legmodel, walkmodel, runmodel);

	float px = data.x;
	float py = data.y;
	float pz = data.z;
	float rx = data.r;
	if( RandomFlag == true ){
		px += 0.001f*GetRand(100*2) - 0.1f;
		pz += 0.001f*GetRand(100*2) - 0.1f;
		rx += DegreeToRadian(0.01f)*GetRand(80*2) - DegreeToRadian(0.8f);
	}

	for(int j=0; j<MAX_HUMAN; j++){
		if( HumanIndex[j].GetEnableFlag() == false ){
			//
			HumanIndex[j].SetPosData(px, py, pz, rx);
			HumanIndex[j].SetParamData(infodata.p2, data.id, data.p4, infodata.p3, true);
			if( GetHumanFlag == 0 ){
				int id = Resource->GetHumanTexture(infodata.p2);
				if( id == -1 ){
					//eNX`擾łȂ΁AtbVtOLɂĖt[擾
					Human_FrameTextureRefresh[j] = true;
				}

				HumanIndex[j].SetTexture(id);
				HumanIndex[j].SetModel(upmodel[ HumanParam.model ], armmodel, legmodel, walkmodel, runmodel);
			}
			else{
#ifdef ENABLE_BUG_HUMANWEAPON
				HumanIndex[j].SetTexture(d3dg->GetMapTextureID(0));
				HumanIndex[j].SetModel(upmodel[0], armmodel, legmodel, walkmodel, runmodel);
#endif
			}
			HumanIndex[j].SetEnableFlag(true);
			Humanindexid = j;
			break;
		}
	}

	//łȂG[ƂĕԂ
	if( Humanindexid == -1 ){
		return -1;
	}

	//l̐ݒf[^ǂ߂Ă΁`
	if( GetHumanFlag == 0 ){
		//BzƂĒǉ
		Weaponindexid = AddVisualWeaponIndex(HumanParam.Weapon[0], true);
		if( Weaponindexid != -1 ){
			//ΔzɋL^
			Weapon[0] = &WeaponIndex[Weaponindexid];
		}
		if( data.p1 == 1 ){
			//ẢzƂĒǉ
			Weaponindexid = AddVisualWeaponIndex(HumanParam.Weapon[1], true);
			if( Weaponindexid != -1 ){
				//ΔzɋL^
				Weapon[1] = &WeaponIndex[Weaponindexid];
			}
		}

		//lɎ
		HumanIndex[Humanindexid].SetWeapon(Weapon, lnbslist, nbslist);
	}

	//CtO
	Human_ShotFlag[Humanindexid] = false;

	//vC[Ȃ΁AԍL^
	if( ( (data.p1 == 1)||(data.p1 == 6) )&&(data.p4 == 0) ){
		Player_HumanID = Humanindexid;
	}

	return Humanindexid;
}

//! @brief lǉiQ[pj
//! @param px XW
//! @param py YW
//! @param pz ZW
//! @param rx X
//! @param paramID ޔԍ
//! @param TeamID `[ԍ
//! @param WeaponID ޔԍ̔zivfFTOTAL_HAVEWEAPONj
//! @param param4 4p[^[
//! @param RandomFlag _␳Lɂ
//! @return Ff[^ԍi0ȏj@sF-1
//! @attention Ȑl̎ޔԍw肳ꂽꍇ ʏ́Fl oꂵ܂BeNX`̓}bveNX`0ԂgpAHP 0 wij܂B
int ObjectManager::AddHumanIndex(float px, float py, float pz, float rx, int paramID, int TeamID, int WeaponID[], signed short int param4, bool RandomFlag)
{
	int GetHumanFlag;
	HumanParameter HumanParam;
	GetHumanFlag = GameParamInfo->GetHuman(paramID, &HumanParam);
	int Humanindexid = -1;
	int Weaponindexid = -1;
	class weapon *Weapon[TOTAL_HAVEWEAPON];
	int lnbslist[TOTAL_HAVEWEAPON];
	int nbslist[TOTAL_HAVEWEAPON];
	for(int i=0; i<TOTAL_HAVEWEAPON; i++){
		Weapon[i] = NULL;
		lnbslist[i] = 0;
		nbslist[i] = 0;
	}

#ifndef ENABLE_BUG_HUMANWEAPON
	if( GetHumanFlag != 0 ){
		return -1;
	}
#endif

	//l̃fԍ擾
	int upmodel[TOTAL_UPMODE];
	int armmodel[TOTAL_ARMMODE];
	int legmodel;
	int walkmodel[TOTAL_WALKMODE];
	int runmodel[TOTAL_RUNMODE];
	Resource->GetHumanModel(upmodel, armmodel, &legmodel, walkmodel, runmodel);

	if( RandomFlag == true ){
		px += 0.001f*GetRand(100*2) - 0.1f;
		pz += 0.001f*GetRand(100*2) - 0.1f;
		rx += DegreeToRadian(0.01f)*GetRand(80*2) - DegreeToRadian(0.8f);
	}

	for(int i=0; i<MAX_HUMAN; i++){
		if( HumanIndex[i].GetEnableFlag() == false ){
			//
			HumanIndex[i].SetPosData(px, py, pz, rx);
			HumanIndex[i].SetParamData(paramID, -1, param4, TeamID, true);
			if( GetHumanFlag == 0 ){
				int id = Resource->GetHumanTexture(paramID);
				if( id == -1 ){
					//eNX`擾łȂ΁AtbVtOLɂĖt[擾
					Human_FrameTextureRefresh[i] = true;
				}

				HumanIndex[i].SetTexture(id);
				HumanIndex[i].SetModel(upmodel[ HumanParam.model ], armmodel, legmodel, walkmodel, runmodel);
			}
			else{
#ifdef ENABLE_BUG_HUMANWEAPON
				HumanIndex[i].SetTexture(d3dg->GetMapTextureID(0));
				HumanIndex[i].SetModel(upmodel[0], armmodel, legmodel, walkmodel, runmodel);
#endif
			}
			HumanIndex[i].SetEnableFlag(true);
			Humanindexid = i;
			break;
		}
	}

	//łȂG[ƂĕԂ
	if( Humanindexid == -1 ){
		return -1;
	}

	if( GetHumanFlag == 0 ){
		//zǉ
		for(int i=0; i<TOTAL_HAVEWEAPON; i++){
			Weaponindexid = AddVisualWeaponIndex(WeaponID[i], true);
			if( Weaponindexid != -1 ){
				//ΔzɋL^
				Weapon[i] = &WeaponIndex[Weaponindexid];
			}
		}

		//lɎ
		HumanIndex[Humanindexid].SetWeapon(Weapon, lnbslist, nbslist);
	}

	//O֌W̏
	int player_teamid;
	HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);
	ObjectLog->AddHuman(Humanindexid, TeamID, player_teamid);

	return Humanindexid;
}

//! @brief ǉ
//! @param data ̃|Cgf[^@ipointdata\́j
//! @return Ff[^ԍi0ȏj@sF-1
int ObjectManager::AddWeaponIndex(pointdata data)
{
	//ԍƒeo^
	int WeaponID = data.p2;
	int nbs = (unsigned char)data.p3;

	//_ȕȂ
	if( data.p1 == 7 ){
		WeaponParameter WeaponParam;

		if( GetRand(2) == 0 ){
			if( GameParamInfo->GetWeapon(data.p2, &WeaponParam) == 1 ){ return -1; }
			nbs = (int)WeaponParam.nbsmax * TOTAL_WEAPON_AUTOBULLET;		//e㏑
		}
		else{
			if( GameParamInfo->GetWeapon(data.p3, &WeaponParam) == 1 ){ return -1; }
			WeaponID = data.p3;						//ԍ㏑
			nbs = (int)WeaponParam.nbsmax * TOTAL_WEAPON_AUTOBULLET;		//e㏑
		}
	}

	//NONEȂΎs
	if( WeaponID == ID_WEAPON_NONE ){ return -1; }

	//fƃeNX`擾
	int model, texture;
	if( Resource->GetWeaponModelTexture(WeaponID, &model, &texture) == 1 ){
		return -1;
	}

	//ݒl擾
	WeaponParameter WeaponParam;
	if( GameParamInfo->GetWeapon(WeaponID, &WeaponParam) == 1 ){ return -1; }

	for(int i=0; i<MAX_WEAPON; i++){
		if( WeaponIndex[i].GetEnableFlag() == false ){
			//
			WeaponIndex[i].SetPosData(data.x, data.y, data.z, data.r);
			WeaponIndex[i].SetParamData(WeaponID, 0, nbs, true);
			WeaponIndex[i].SetModel(model, WeaponParam.size);
			WeaponIndex[i].SetTexture(texture);
			WeaponIndex[i].SetEnableFlag(true);
			WeaponIndex[i].RunReload();
			return i;
		}
	}
	return -1;
}

//! @brief zǉ
//! @param WeaponID ̎ޔԍ
//! @param loadbullet e[h
//! @return Ff[^ԍi0ȏj@sF-1
//! @attention lŏ珊Ă镐Af[^iIuWFNgjƂĒǉ邽߂̊֐łB
//! @attention P̂ƂĔzuꍇ́AAddWeaponIndex()֐gpĂB
int ObjectManager::AddVisualWeaponIndex(int WeaponID, bool loadbullet)
{
	//NONEȂΎs
	if( WeaponID == ID_WEAPON_NONE ){ return -1; }

	//擾
	WeaponParameter WeaponParam;
	if( GameParamInfo->GetWeapon(WeaponID, &WeaponParam) == 1 ){ return -1; }

	//fƃeNX`擾
	int model, texture;
	if( Resource->GetWeaponModelTexture(WeaponID, &model, &texture) == 1 ){
		return -1;
	}

	for(int i=0; i<MAX_WEAPON; i++){
		if( WeaponIndex[i].GetEnableFlag() == false ){
			//
			WeaponIndex[i].SetPosData(0.0f, 0.0f, 0.0f, 0.0f);
			if( loadbullet == false ){
				WeaponIndex[i].SetParamData(WeaponID, 0, 0, true);
			}
			else{
				WeaponIndex[i].SetParamData(WeaponID, WeaponParam.nbsmax, WeaponParam.nbsmax * TOTAL_WEAPON_AUTOBULLET, true);
			}
			WeaponIndex[i].SetModel(model, WeaponParam.size);
			WeaponIndex[i].SetTexture(texture);
			WeaponIndex[i].SetEnableFlag(true);
			return i;
		}
	}
	return -1;
}

//! @brief ǉ
//! @param data ̃|Cgf[^@ipointdata\́j
//! @return Ff[^ԍi0ȏj@sF-1
int ObjectManager::AddSmallObjectIndex(pointdata data)
{
	//fƃeNX`擾
	int model, texture;
	if( Resource->GetSmallObjectModelTexture(data.p2, &model, &texture) == 1 ){
		return -1;
	}

	for(int i=0; i<MAX_SMALLOBJECT; i++){
		if( SmallObjectIndex[i].GetEnableFlag() == false ){
			//
			SmallObjectIndex[i].SetPosData(data.x, data.y, data.z, data.r);
			SmallObjectIndex[i].SetParamData(data.p2, data.p4, true);
			SmallObjectIndex[i].SetModel(model, SMALLOBJECT_SCALE);
			SmallObjectIndex[i].SetTexture(texture);
			SmallObjectIndex[i].SetEnableFlag(true);

			//ʒuCtOLȂ΁A}bvƔ
			if( data.p3 !=0 ){
				SmallObjectIndex[i].CollisionMap(CollD);
			}
			return i;
		}
	}
	return -1;
}

//! @brief ǉiQ[pj
//! @param px XW
//! @param py YW
//! @param pz ZW
//! @param rx X
//! @param paramID ޔԍ
//! @param MapColl ʒuCtO
//! @return Ff[^ԍi0ȏj@sF-1
int ObjectManager::AddSmallObjectIndex(float px, float py, float pz, float rx, int paramID, bool MapColl)
{
	//fƃeNX`擾
	int model, texture;
	if( Resource->GetSmallObjectModelTexture(paramID, &model, &texture) == 1 ){
		return -1;
	}

	for(int i=0; i<MAX_SMALLOBJECT; i++){
		if( SmallObjectIndex[i].GetEnableFlag() == false ){
			//
			SmallObjectIndex[i].SetPosData(px, py, pz, rx);
			SmallObjectIndex[i].SetParamData(paramID, 0, true);
			SmallObjectIndex[i].SetModel(model, SMALLOBJECT_SCALE);
			SmallObjectIndex[i].SetTexture(texture);
			SmallObjectIndex[i].SetEnableFlag(true);

			//ʒuCtOLȂ΁A}bvƔ
			if( MapColl == true ){
				SmallObjectIndex[i].CollisionMap(CollD);
			}

			//O֌W̏
			ObjectLog->AddSmallObject(i);

			return i;
		}
	}
	return -1;
}

//! @brief GtFNgǉ
//! @param pos_x XW
//! @param pos_y YW
//! @param pos_z ZW
//! @param move_x Xړ
//! @param move_y Yړ
//! @param move_z Zړ
//! @param addmove_y Yړʂ̉Z
//! @param rotation ]px
//! @param addrotation ]px̉Z
//! @param size `{
//! @param addsize `{̉Z
//! @param texture eNX`̔Fԍ
//! @param brightness 邳
//! @param addbrightness 邳̉Z
//! @param alpha x
//! @param addalpha x̉Z
//! @param settype GtFNg̎ށ@iEffect_Type gݍj
//! @param count `t[
//! @return Ff[^ԍi0ȏj@sF-1
int ObjectManager::AddEffect(float pos_x, float pos_y, float pos_z, float move_x, float move_y, float move_z, float addmove_y, float rotation, float addrotation, float size, float addsize, int texture, int brightness, int addbrightness, float alpha, float addalpha, int settype, int count)
{
	for(int i=0; i<MAX_EFFECT; i++){
		if( EffectIndex[i].GetEnableFlag() == false ){
			EffectIndex[i].SetPosData(pos_x, pos_y, pos_z, 0.0f);
			EffectIndex[i].SetParamData(move_x, move_y, move_z, addmove_y, rotation, addrotation, size, addsize, count, settype);
			EffectIndex[i].SetTexture(texture, brightness, addbrightness, alpha, addalpha);
			EffectIndex[i].SetEnableFlag(true);
			return i;
		}
	}

	return -1;
}

//! @brief ir{[hłȂjGtFNgǉ
//! @param id ubNԍ
//! @param face ʔԍ
//! @param pos_x XW
//! @param pos_y YW
//! @param pos_z ZW
//! @param size `{
//! @param rotation ]px
//! @param texture eNX`̔Fԍ
//! @param brightness 邳
//! @param addbrightness 邳̉Z
//! @param alpha x
//! @param addalpha x̉Z
//! @param count `t[
//! @return Ff[^ԍi0ȏj@sF-1
int ObjectManager::AddMapEffect(int id, int face, float pos_x, float pos_y, float pos_z, float size, float rotation, int texture, int brightness, int addbrightness, float alpha, float addalpha, int count)
{
	blockdata data;
	float vx, vy, vz;
	float rx, ry;

	//ubN̖ʂ̏擾
	BlockData->Getdata(&data, id);
	vx = data.material[face].vx;
	vy = data.material[face].vy;
	vz = data.material[face].vz;

	//px߂
	rx = atan2f(vz, vx)*-1 + (float)M_PI;
	ry = atan2f(vy, sqrtf(vx*vx + vz*vz))*-1;

	//GtFNg쐬
	for(int i=0; i<MAX_EFFECT; i++){
		if( EffectIndex[i].GetEnableFlag() == false ){
			EffectIndex[i].SetPosData(pos_x, pos_y, pos_z, 0.0f);
			EffectIndex[i].SetParamData(0.0f, 0.0f, 0.0f, 0.0f, rotation, 0.0f, size, 0.0f, count, EFFECT_NOBILLBOARD);
			EffectIndex[i].SetRxRy(rx, ry);
			EffectIndex[i].SetTexture(texture, brightness, addbrightness, alpha, addalpha);
			EffectIndex[i].SetEnableFlag(true);
			return i;
		}
	}

	return -1;
}

//! @brief o
//! @param x XW
//! @param y YW
//! @param z ZW
//! @param damage _[W
//! @param flowing ̗L
void ObjectManager::SetHumanBlood(float x, float y, float z, int damage, bool flowing)
{
	int addtype;

	if( flowing == false ){
		addtype = EFFECT_NORMAL;
	}
	else{
		addtype = EFFECT_COLLIDEMAP;
	}

	if( GameConfig.GetBloodFlag() == true ){
		AddEffect(x, y, z, 0.0f, 0.0f, 0.0f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 10.0f, 0.2f, Resource->GetEffectBloodTexture(), 10, 0, 0.5f, -0.04f, addtype, (int)(GAMEFPS * 0.4f));
		if( flowing == true ){
			for(int i=0; i<(damage/10); i++){
				float rx = DegreeToRadian(10)*GetRand(36);
				AddEffect(x + cosf(rx)*1.0f, y + (float)(GetRand(20)-10)/10, z + sinf(rx)*1.0f, cosf(rx)*0.5f, GetRand(5)*0.1f + 0.5f, sinf(rx)*0.5f, -0.09f, DegreeToRadian(10)*GetRand(36), 0.0f, GetRand(40)*0.1f+0.1f, 0.1f, Resource->GetEffectBloodTexture(), 10, 0, 1.0f, 0.0f, addtype, (int)(GAMEFPS * 0.5f));
			}
		}
	}
}

//! @brief lm̓蔻
//! @param in_humanA Ώۂ̐lIuWFNgA
//! @param in_humanB Ώۂ̐lIuWFNgB
//! @return ĂFtrue@ĂȂFfalse
//! @warning in_humanAin_humanBŋʂ͂܂B
//! @warning in_humanAin_humanBł̑gݍ́A1t[Ԃ1xsĂB
//! @attention NX͎IAddPosOrder()pāA݂܂B
bool ObjectManager::CollideHuman(human *in_humanA, human *in_humanB)
{
	if( in_humanA == NULL ){ return false; }
	if( in_humanB == NULL ){ return false; }

	float h1_x, h1_y, h1_z;
	float h2_x, h2_y, h2_z;
	float angle, length;

	//ĂȂASēΔ肵Ȃ
	if( in_humanA->GetEnableFlag() == false ){ return false; }
	if( in_humanB->GetEnableFlag() == false ){ return false; }
	if( in_humanA->GetHP() <= 0 ){ return false; }
	if( in_humanB->GetHP() <= 0 ){ return false; }

	//݂̍W擾
	in_humanA->GetPosData(&h1_x, &h1_y, &h1_z, NULL);
	in_humanB->GetPosData(&h2_x, &h2_y, &h2_z, NULL);

	//~̓蔻
	if( CollideCylinder(h1_x, h1_y, h1_z, HUMAN_HUMANCOLLISION_R, HUMAN_HEIGHT-0.5f, h2_x, h2_y, h2_z, HUMAN_HUMANCOLLISION_R, HUMAN_HEIGHT-0.5f, &angle, &length) == true ){
		//߂荞񂾕o
		in_humanA->AddPosOrder(angle, 0.0f, length/2);
		in_humanB->AddPosOrder(angle + (float)M_PI, 0.0f, length/2);
		return true;
	}

	return false;
}

//! @brief e̓蔻Ə
//! @param in_bullet Ώۂ̒eIuWFNg
//! @return Ftrue@ĂȂFfalse
//! @attention sΏۂ́u}bvvuliE㔼gEgjvuvłB
//! @attention Ɍ炸A_[WvZʉĐ܂ňтčs܂B
bool ObjectManager::CollideBullet(bullet *in_bullet)
{
	if( in_bullet == NULL ){ return false; }

	//gpĂȂeۂȂ΁AɕԂB
	if( in_bullet->GetEnableFlag() == false ){ return false; }

	int objectID;
	float bx, by, bz;
	float brx, bry;
	int attacks;
	int penetration;
	int speed;
	int teamid;
	int humanid;
	float ontargetcnt;
	float vx, vy, vz;
	bool CollideFlag = false;
	int mapflag = 0;		//0FubNȂA1FubN薢A2FubN蔻ς

	float Dist;

	//eIuWFNg̃f[^ԍ擾
	objectID = GetBulletObjectID(in_bullet);

	//eۂ̍W擾AxNgZoB
	in_bullet->GetPosData(&bx, &by, &bz, &brx, &bry);
	in_bullet->GetParamData(&attacks, &penetration, &speed, &teamid, &humanid, &ontargetcnt);
	vx = cosf(brx)*cosf(bry);
	vy = sinf(bry);
	vz = sinf(brx)*cosf(bry);

	//}bvƂ̓蔻ieɃubN邩j
	if( CollD->CheckALLBlockIntersectRay(bx, by, bz, vx, vy, vz, NULL, NULL, &Dist, (float)speed) == true ){
		mapflag = 1;
	}

	// BULLET_SPEEDSCALE萔 ̕\Zo
	int maxcnt = (int)ceilf(speed / BULLET_SPEEDSCALE);

	// BULLET_SPEEDSCALE萔 ̕\œ蔻
	for(int cnt=0; cnt<maxcnt; cnt++){
		//ђʗ͂cĂȂΒe͖ɂi 1ځj
		if( penetration < 0 ){
			in_bullet->SetEnableFlag(false);
			mapflag = 0;
			break;
		}

		float bvx, bvy, bvz;
		bvx = bx + vx * BULLET_SPEEDSCALE * cnt;
		bvy = by + vy * BULLET_SPEEDSCALE * cnt;
		bvz = bz + vz * BULLET_SPEEDSCALE * cnt;

		//lƂ̓蔻
		for(int i=0; i<MAX_HUMAN; i++){
			//̐lgCeȂ珈ȂB
			if( i == humanid ){ continue; }

			//gpĂȂÂȂΏȂB
			if( HumanIndex[i].GetEnableFlag() == false ){ continue; }
			//if( HumanIndex[i].GetDeadFlag() == true ){ continue; }
			if( HumanIndex[i].GetHP() <= 0 ){ continue; }

			//ɓlȂAȂB
			if( BulletObj_HumanIndex[objectID].GetIndexFlag(i) == true ){ continue; }

			//W擾
			float ox, oy, oz;
			int h_teamid;
			HumanIndex[i].GetPosData(&ox, &oy, &oz, NULL);
			HumanIndex[i].GetParamData(NULL, NULL, NULL, &h_teamid);

			if( FriendlyFire[i] == false ){
				//`[ԍijȂ珈Ȃ
				if( h_teamid == teamid ){ continue; }
			}

			//̓蔻
			if( CollideCylinderInside(ox, oy + HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H, oz, HUMAN_BULLETCOLLISION_HEAD_R, HUMAN_BULLETCOLLISION_HEAD_H, bvx, bvy, bvz) == true ){
				//lɓ
				HitBulletHuman(i, 0, bvx, bvy, bvz, brx, attacks, humanid, ontargetcnt);

				//ΐlpXgݒixAŌɓlݒj
				BulletObj_HumanIndex[objectID].Init();
				BulletObj_HumanIndex[objectID].SetIndexFlag(i);

				//U͂Ɗђʗ͂vZ
				attacks = (int)((float)attacks * 0.5f);
				penetration -= 1;

				CollideFlag = true;
			}

			//㔼gƓ蔻
			if( CollideCylinderInside(ox, oy + HUMAN_BULLETCOLLISION_LEG_H, oz, HUMAN_BULLETCOLLISION_UP_R, HUMAN_BULLETCOLLISION_UP_H, bvx, bvy, bvz) == true ){
				//lɓ
				HitBulletHuman(i, 1, bvx, bvy, bvz, brx, attacks, humanid, ontargetcnt);

				//ΐlpXgݒixAŌɓlݒj
				BulletObj_HumanIndex[objectID].Init();
				BulletObj_HumanIndex[objectID].SetIndexFlag(i);

				//U͂Ɗђʗ͂vZ
				attacks = (int)((float)attacks * 0.6f);
				penetration -= 1;

				CollideFlag = true;
			}

			//Ɠ蔻
			if( CollideCylinderInside(ox, oy, oz, HUMAN_BULLETCOLLISION_LEG_R, HUMAN_BULLETCOLLISION_LEG_H, bvx, bvy, bvz) == true ){
				//lɓ
				HitBulletHuman(i, 2, bvx, bvy, bvz, brx, attacks, humanid, ontargetcnt);

				//ΐlpXgݒixAŌɓlݒj
				BulletObj_HumanIndex[objectID].Init();
				BulletObj_HumanIndex[objectID].SetIndexFlag(i);

				//U͂Ɗђʗ͂vZ
				attacks = (int)((float)attacks * 0.7f);
				penetration -= 1;

				CollideFlag = true;
			}
		}

		//ђʗ͂cĂȂΒe͖ɂi 2ځj
		if( penetration < 0 ){
			in_bullet->SetEnableFlag(false);
			mapflag = 0;
			break;
		}

		//Ƃ̓蔻
		for(int i=0; i<MAX_SMALLOBJECT; i++){
			//gpĂȂΏȂ
			if( SmallObjectIndex[i].GetEnableFlag() == false ){ continue; }

			//W擾
			float ox, oy, oz;
			int id;
			float decide;
			SmallObjectIndex[i].GetPosData(&ox, &oy, &oz, NULL);
			SmallObjectIndex[i].GetParamData(&id, NULL);

			//蔻̑傫擾
			if( (TOTAL_PARAMETERINFO_SMALLOBJECT <= id)&&(id <= (TOTAL_PARAMETERINFO_SMALLOBJECT + MAX_ADDSMALLOBJECT -1)) ){
				decide = (float)MIFdata->GetAddSmallobjectDecide(id - TOTAL_PARAMETERINFO_SMALLOBJECT)*SMALLOBJECT_COLLISIONSCALE;
			}
			else{
				SmallObjectParameter Param;
				GameParamInfo->GetSmallObject(id, &Param);
				decide = (float)Param.decide*SMALLOBJECT_COLLISIONSCALE;
			}

			//蔻
			if( CollideSphereInside(ox, oy, oz, decide, bvx, bvy, bvz) == true ){
				//ɓ
				int attack2 = (int)(floorf( (float)attacks * 0.25f ));
				HitBulletSmallObject(i, bvx, bvy, bvz, attack2, teamid);

				//U͂vZ
				attacks = (int)(floorf( (float)attacks * 0.7f ));

				CollideFlag = true;
			}
		}

		if( mapflag > 0 ){
			//}bvƂ̓蔻
			if( CollD->CheckALLBlockInside(bvx, bvy, bvz) == true ){
				//e}bvɓ
				HitBulletMap(bx + vx*(Dist-1.0f), by + vy*(Dist-1.0f), bz + vz*(Dist-1.0f), teamid);

				//U͂Ɗђʗ͂vZ
				penetration -= 1;
				if( penetration >= 0 ){
					attacks = (int)((float)attacks * 0.6f);
				}

				mapflag = 2;
				CollideFlag = true;
			}
		}
	}

	//eɃubNɂւ炸AubNƓ蔻sĂȂȂ
	//@iubNj
	if( mapflag == 1 ){
		//e}bvɓ
		HitBulletMap(bx + vx*(Dist-1.0f), by + vy*(Dist-1.0f), bz + vz*(Dist-1.0f), teamid);

		//U͂vZ
		if( penetration > 0 ){
			attacks = (int)((float)attacks * 0.75f);
		}
		else{
			attacks = (int)((float)attacks * 0.55f);
		}

		CollideFlag = true;
	}

	//ݒKpiɍÚEђʗ́j
	in_bullet->SetParamData(attacks, penetration, speed, teamid, humanid, ontargetcnt, false);

	return CollideFlag;
}

//! @brief e}bvɓ
//! @param x eXW
//! @param y eYW
//! @param z eZW
//! @param teamID ˌ̃`[ԍ
void ObjectManager::HitBulletMap(float x, float y, float z, int teamID)
{
	//GtFNgij`
	AddEffect(x, y, z, 0.0f, 0.05f, 0.0f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 3.0f, 0.24f, Resource->GetEffectSmokeTexture(), 20, 0, 0.63f, -0.03f, EFFECT_NORMAL, 20);

	//ʉĐ
	GameSound->HitMap(x, y, z, teamID);
}

//! @brief elɓ
//! @param HitHuman_id l̔ԍ
//! @param Hit_id eӏiF0@́F1@F2j
//! @param x eXW
//! @param y eYW
//! @param z eZW
//! @param brx px
//! @param attacks U
//! @param Shothuman_id ˂l̔ԍ
//! @param ontargetcnt ̃JEg
void ObjectManager::HitBulletHuman(int HitHuman_id, int Hit_id, float x, float y, float z, float brx, int attacks, int Shothuman_id, float ontargetcnt)
{
	int Shothuman_TeamID;
	int HitHuman_TeamID;
	int hp_old;
	int damage = 0;
	int paramid;
	HumanParameter Paraminfo;
	bool NotRobot;

	//gpĂȂÂȂΏȂB
	if( HumanIndex[HitHuman_id].GetEnableFlag() == false ){ return; }
	if( HumanIndex[HitHuman_id].GetDeadFlag() == true ){ return; }

	//ˌƑΏېl̃`[ԍ擾
	HumanIndex[Shothuman_id].GetParamData(NULL, NULL, NULL, &Shothuman_TeamID);
	HumanIndex[HitHuman_id].GetParamData(NULL, NULL, NULL, &HitHuman_TeamID);

	//_[WvZOHP擾
	hp_old = HumanIndex[HitHuman_id].GetHP();

	//lɃ_[WƏՌ^
	if( Hit_id == 0 ){ HumanIndex[HitHuman_id].HitBulletHead(attacks); }
	if( Hit_id == 1 ){ HumanIndex[HitHuman_id].HitBulletUp(attacks); }
	if( Hit_id == 2 ){ HumanIndex[HitHuman_id].HitBulletLeg(attacks); }
	HumanIndex[HitHuman_id].AddPosOrder(brx, 0.0f, 1.0f);

#ifdef ENABLE_BUG_TEAMID
	bool flag = true;

	//`[ԍA`[ԍ傫ȂAtO
	if( (HitHuman_TeamID < 0)&&(Shothuman_TeamID < 0) ){
		if( HitHuman_TeamID < Shothuman_TeamID ){
			flag = false;
		}
	}

	if( flag == true ){
		HumanIndex[HitHuman_id].SetHitFlag(brx*-1 - (float)M_PI/2);
	}
#else
	HumanIndex[HitHuman_id].SetHitFlag(brx*-1 - (float)M_PI/2);
#endif

	//{bgǂ
	HumanIndex[HitHuman_id].GetParamData(&paramid, NULL, NULL, NULL);
	GameParamInfo->GetHuman(paramid, &Paraminfo);
	if( Paraminfo.type == 1 ){
		NotRobot = false;
	}
	else{
		NotRobot = true;
	}

	//GtFNgij`
	if( Hit_id == 0 ){ damage = (int)((float)attacks * HUMAN_DAMAGE_HEAD); }
	if( Hit_id == 1 ){ damage = (int)((float)attacks * HUMAN_DAMAGE_UP); }
	if( Hit_id == 2 ){ damage = (int)((float)attacks * HUMAN_DAMAGE_LEG); }
	SetHumanBlood(x, y, z, damage, NotRobot);

	//ʉĐ
	GameSound->HitHuman(x, y, z, Hit_id, Shothuman_TeamID);

	if( (0 <= Shothuman_id)&&(Shothuman_id < MAX_HUMAN) ){
		//e𔭎˂l̐ʂɉZ
		Human_ontarget[Shothuman_id] += ontargetcnt;
		if( Hit_id == 0 ){ Human_headshot[Shothuman_id] += 1; }
		if( (hp_old > 0)&&(HumanIndex[HitHuman_id].GetHP() <= 0) ){
			//_[WvZO hp>0 łAvZ hp <= 0 ȂA񎀖SB
			Human_kill[Shothuman_id] += 1;
		}
	}

	//O֌W̏
	if( (hp_old > 0)&&(HumanIndex[HitHuman_id].GetHP() <= 0) ){
		//_[WvZO hp>0 łAvZ hp <= 0 ȂA񎀖SB

		int player_teamid;
		HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);

		ObjectLog->DiedHuman(Shothuman_id, HitHuman_id, Shothuman_TeamID, HitHuman_TeamID, player_teamid);
	}
}

//! @brief eɓ
//! @param HitSmallObject_id ̔ԍ
//! @param x eXW
//! @param y eYW
//! @param z eZW
//! @param attacks U
//! @param teamID ˌ̃`[ԍ
void ObjectManager::HitBulletSmallObject(int HitSmallObject_id, float x, float y, float z, int attacks, int teamID)
{
	int hp;

	//gpĂȂΏȂB
	if( SmallObjectIndex[HitSmallObject_id].GetEnableFlag() == false ){ return; }

	//̗͂ȂΏȂ
	hp = SmallObjectIndex[HitSmallObject_id].GetHP();
	if( hp <= 0 ){ return; }

	//Ƀ_[W^
	SmallObjectIndex[HitSmallObject_id].HitBullet(attacks);

	//GtFNgij`
	//AddEffect(x, y, z, 0.01f*(GetRand(101)-50), 0.01f*(GetRand(101)-50), 0.01f*(GetRand(101)-50), 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 1.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 0.625f, -0.047f, EFFECT_NORMAL, 255);
	//AddEffect(x, y, z, 0.01f*(GetRand(101)-50), 0.01f*(GetRand(101)-50), 0.01f*(GetRand(101)-50), 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 1.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 0.625f, -0.047f, EFFECT_NORMAL, 255);
	AddEffect(x, y, z, 0.0f, 0.0f, 0.0f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 1.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 0.625f, -0.047f, EFFECT_NORMAL, 255);
	AddEffect(x, y, z, 0.0f, 0.01f*(GetRand(10)-5), 0.0f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, 1.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 0.625f, -0.047f, EFFECT_NORMAL, 255);

	//ʉĐ
	int id;
	SmallObjectIndex[HitSmallObject_id].GetParamData(&id, NULL);
	GameSound->HitSmallObject(x, y, z, id, teamID);

	//O֌W̏
	hp = SmallObjectIndex[HitSmallObject_id].GetHP();
	if( hp <= 0 ){
		//_[WvZO hp>0 łAvZ hp <= 0 ȂAj󂳂ꂽB
		ObjectLog->BreakSmallObject(HitSmallObject_id);
	}
}

//! @brief ֒ẽ_[WƏ
//! @param in_grenade Ώۂ̎֒eIuWFNg
//! @return Ftrue@ĂȂFfalse
//! @attention sΏۂ́ulvƁuvłB
//! @attention _[WɌ炸A_[WvZʉĐ܂ňтčs܂B
bool ObjectManager::GrenadeExplosion(grenade *in_grenade)
{
	if( in_grenade == NULL ){ return false; }

	bool returnflag = false;

	//W擾
	float gx, gy, gz;
	int teamid, humanid;
	in_grenade->GetPosData(&gx, &gy, &gz, NULL, NULL);
	in_grenade->GetParamData(NULL, &teamid, &humanid, NULL);

	//lɔ̓蔻
	for(int i=0; i<MAX_HUMAN; i++){
		//ĂȂÂȂΏȂB
		if( HumanIndex[i].GetEnableFlag() == false ){ continue; }
		if( HumanIndex[i].GetDeadFlag() == true ){ continue; }

		int HitHuman_TeamID;
		float hx, hy, hz;
		float x, y, z, r;

		//l̍W擾AvZ
		HumanIndex[i].GetPosData(&hx, &hy, &hz, NULL);
		x = hx - gx;
		y = hy - gy;
		z = hz - gz;
		r = sqrtf(x*x + y*y + z*z);

		//100.0艓ΌvZȂ
		if( r > MAX_DAMAGE_GRENADE_DISTANCE + HUMAN_HEIGHT ){ continue; }

		float dummy = 0.0f;
		int total_damage = 0;
		int damage;

		//ɓ蔻
		y = hy + 2.0f - gy;
		r = sqrtf(x*x + y*y + z*z);
		//ubNՂĂȂ΁@iCœ蔻sAĂȂ΁j
		if( CollD->CheckALLBlockIntersectRay(gx, gy, gz, x/r, y/r, z/r, NULL, NULL, &dummy, r) == false ){
			//_[WʂvZ
			damage = HUMAN_DAMAGE_GRENADE_LEG - (int)((float)HUMAN_DAMAGE_GRENADE_LEG/MAX_DAMAGE_GRENADE_DISTANCE * r);
			if( damage > 0 ){
				total_damage += damage;
			}
		}

		//ɓ蔻
		y = hy + (HUMAN_HEIGHT-2.0f) - gy;
		r = sqrtf(x*x + y*y + z*z);
		//ubNՂĂȂ΁@iCœ蔻sAĂȂ΁j
		if( CollD->CheckALLBlockIntersectRay(gx, gy, gz, x/r, y/r, z/r, NULL, NULL, &dummy, r) == false ){
			//_[WʂvZ
			damage = HUMAN_DAMAGE_GRENADE_HEAD - (int)((float)HUMAN_DAMAGE_GRENADE_HEAD/MAX_DAMAGE_GRENADE_DISTANCE * r);
			if( damage > 0 ){
				total_damage += damage;
			}
		}

		if( total_damage > 0 ){
			int hp_old;

			//_[WvZOHP擾
			hp_old = HumanIndex[i].GetHP();

			//_[W𔽉f
			HumanIndex[i].HitGrenadeExplosion(total_damage);

			HumanIndex[i].GetParamData(NULL, NULL, NULL, &HitHuman_TeamID);

			float y2;
			float arx, ary;

			if( (0 <= humanid)&&(humanid < MAX_HUMAN) ){
				//|Ă΁A˂l̐ʂɉZ
				if( HumanIndex[i].GetHP() <= 0 ){ Human_kill[humanid] += 1; }
			}

			//GtFNgij`
			SetHumanBlood(hx, hy+15.0f, hz, total_damage, false);

			//lƎ֒e̋Zo
			x = gx - hx;
			y = gy - (hy + 1.0f);
			z = gz - hz;

			//px߂
			arx = atan2f(z, x);

			if( sinf(atan2f(y, sqrtf(x*x + z*z))) < 0.0f ){		//ɔԂȂApxvZ
				y2 = gy - (hy + HUMAN_HEIGHT);
				ary = atan2f(y2, sqrtf(x*x + z*z)) + (float)M_PI;
			}
			else{		//ɔԂȂApx͖BiŒnʂɂ߂荞ނ̂h~j
				ary = 0.0f;
			}

#ifdef ENABLE_BUG_TEAMID
			bool flag = true;

			//`[ԍA`[ԍ傫ȂAtO
			if( (HitHuman_TeamID < 0)&&(teamid < 0) ){
				if( HitHuman_TeamID < teamid ){
					flag = false;
				}
			}

			if( flag == true ){
				HumanIndex[i].SetHitFlag(arx*-1 + (float)M_PI/2);
			}
#else
			HumanIndex[i].SetHitFlag(arx*-1 + (float)M_PI/2);
#endif

			//ɂ镗
			HumanIndex[i].AddPosOrder(arx, ary, 2.2f/MAX_DAMAGE_GRENADE_DISTANCE * (MAX_DAMAGE_GRENADE_DISTANCE - sqrtf(x*x + y*y + z*z)));

			//O֌W̏
			if( (hp_old > 0)&&(HumanIndex[i].GetHP() <= 0) ){
				//_[WvZO hp>0 łAvZ hp <= 0 ȂA񎀖SB

				int player_teamid;
				HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);

				ObjectLog->DiedHuman(humanid, i, teamid, HitHuman_TeamID, player_teamid);
			}

			returnflag = true;
		}
	}

	//ɔ̓蔻
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		//gpĂΏȂ
		if( SmallObjectIndex[i].GetEnableFlag() == false ){ continue; }

		float sx, sy, sz;
		float x, y, z, r;

		//̍W擾AvZ
		SmallObjectIndex[i].GetPosData(&sx, &sy, &sz, NULL);
		x = sx - gx;
		y = sy - gy;
		z = sz - gz;
		r = sqrtf(x*x + y*y + z*z);

		//100.0艓ΌvZȂ
		if( r > MAX_DAMAGE_GRENADE_DISTANCE ){ continue; }

		float dummy = 0.0f;

		//ubNՂĂȂ΁@iCœ蔻sAĂȂ΁j
		if( CollD->CheckALLBlockIntersectRay(gx, gy, gz, x/r, y/r, z/r, NULL, NULL, &dummy, r) == false ){
			int id, damage;
			SmallObjectIndex[i].GetParamData(&id, NULL);

			//_[WʂvZAf
			damage = SMALLOBJECT_DAMAGE_GRENADE - (int)((float)SMALLOBJECT_DAMAGE_GRENADE/MAX_DAMAGE_GRENADE_DISTANCE * r);
			SmallObjectIndex[i].HitGrenadeExplosion(damage);

			//ʉ𔭂
			GameSound->HitSmallObject(sx, sy, sz, id, teamid);

			//O֌W̏
			if( SmallObjectIndex[i].GetHP() <= 0 ){
				//_[WvZO hp>0 łAvZ hp <= 0 ȂAj󂳂ꂽB
				ObjectLog->BreakSmallObject(i);
			}

			returnflag = true;
		}
	}

	//GtFNgitbVj̕`
	AddEffect(gx, gy, gz, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 40.0f, 0.0f, Resource->GetEffectMflashTexture(), 0, 0, 0.63f, 0.0f, EFFECT_NORMAL, 2);

	//GtFNgij̕`
	AddEffect(gx, gy, gz, 0.0f, 0.1f, 0.0f, 0.0f,
					DegreeToRadian(10)*GetRand(36), 0.0f, 36.0f, 0.2f, Resource->GetEffectSmokeTexture(), 220, 0, 0.6f, -0.01f, EFFECT_NORMAL, 50);
	AddEffect(gx + ((float)GetRand(7)/10-0.3f), gy + ((float)GetRand(5)/10-0.2f), gz + ((float)GetRand(7)/10-0.3f), (float)GetRand(5)/10-0.2f, (float)GetRand(3)/10, (float)GetRand(5)/10-0.2f, 0.0f,
					DegreeToRadian(10)*GetRand(36), 0.0f, ((float)GetRand(11)+25.0f)*2, 0.3f, Resource->GetEffectSmokeTexture(), 0, 0, 0.47f, -0.004f, EFFECT_NORMAL, 120);
	AddEffect(gx + ((float)GetRand(13)/10-0.6f), gy + ((float)GetRand(11)/10-0.5f), gz + ((float)GetRand(13)/10-0.6f), (float)GetRand(7)/10-0.3f, (float)GetRand(7)/10-0.3f, (float)GetRand(7)/10-0.3f, 0.0f,
					DegreeToRadian(10)*GetRand(36), DegreeToRadian(1), ((float)GetRand(21)+20.0f)*2, 0.2f, Resource->GetEffectSmokeTexture(), 0, 0, 0.4f, -0.004f, EFFECT_NORMAL, 100);
	AddEffect(gx + ((float)GetRand(13)/10-0.6f), gy + ((float)GetRand(11)/10-0.5f), gz + ((float)GetRand(13)/10-0.6f), (float)GetRand(7)/10-0.3f, (float)GetRand(7)/10-0.3f, (float)GetRand(7)/10-0.3f, 0.0f,
					DegreeToRadian(10)*GetRand(36), DegreeToRadian(-1), ((float)GetRand(21)+20.0f)*2, 0.2f, Resource->GetEffectSmokeTexture(), 0, 0, 0.4f, -0.004f, EFFECT_NORMAL, 100);

	//ʉĐ
	GameSound->GrenadeExplosion(gx, gy, gz, teamid);

	return returnflag;
}

//! @brief |ꂽۂ̃GtFNgݒ
//! @param in_human Ώۂ̐lIuWFNg
void ObjectManager::DeadEffect(human *in_human)
{
	if( in_human == NULL ){ return; }

	int paramid;
	HumanParameter data;
	bool robot;

	in_human->GetParamData(&paramid, NULL, NULL, NULL);

	if( GameParamInfo->GetHuman(paramid, &data) != 0 ){ return; }

	if( data.type == 1 ){
		robot = true;
	}
	else{
		robot = false;
	}

	//{bgȂ
	if( robot == true ){
		float hx, hy, hz, hrx, hry;

		//ep[^[擾
		in_human->GetPosData(&hx, &hy, &hz, &hrx);
		hry = in_human->GetDeadRy();

		//ӂ̍WZo
		hx += cosf(hrx*-1 - (float)M_PI/2) * sinf(hry) * HUMAN_HEIGHT/2;
		hz += sinf(hrx*-1 - (float)M_PI/2) * sinf(hry) * HUMAN_HEIGHT/2;

		//GtFNgij̕`
		float rnd = DegreeToRadian(10)*GetRand(18);
		AddEffect(hx+1.0f, hy+1.0f, hz+1.0f, 0.1f, 0.05f, 0.1f, 0.0f, rnd, 0, GetRand(30)*0.1f+30.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 1.0f, -1.0f / (GAMEFPS * 3), EFFECT_NORMAL, (int)GAMEFPS * 3);
		AddEffect(hx+1.0f, hy-1.0f, hz-1.0f, 0.2f, 0.05f, -0.2f, 0.0f, rnd*-1, 0, GetRand(30)*0.1f+30.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 1.0f, -1.0f / (GAMEFPS * 3), EFFECT_NORMAL, (int)GAMEFPS * 3);
		AddEffect(hx-1.0f, hy-1.0f, hz-1.0f, -0.3f, 0.05f, -0.3f, 0.0f, rnd, 0, GetRand(30)*0.1f+30.0f, 0.1f, Resource->GetEffectSmokeTexture(), 10, 0, 1.0f, -1.0f / (GAMEFPS * 3), EFFECT_NORMAL, (int)GAMEFPS * 3);
	}
}

//! }bvɌt邩
//! @param in_effect Ώۂ̃GtFNgIuWFNg
//! @param id tubNԍ󂯎|C^
//! @param face tʔԍ󂯎|C^
//! @param pos_x XW󂯎|C^
//! @param pos_y YW󂯎|C^
//! @param pos_z ZW󂯎|C^
//! @return tFtrue@tȂFfalse
bool ObjectManager::CollideBlood(effect *in_effect, int *id, int *face, float *pos_x, float *pos_y, float *pos_z)
{
	if( in_effect == NULL ){ return false; }
	if( (id == NULL)||(face == NULL) ){ return false; }
	if( (pos_x == NULL)||(pos_y == NULL)||(pos_z == NULL) ){ return false; }

	//ȃGtFNgȂΏȂ
	if( in_effect->GetEnableFlag() == false ){ return false; }
	if( in_effect->GetCollideMapFlag() == false ){ return false; }
	if( in_effect->GetTextureID() != Resource->GetEffectBloodTexture() ){ return false; }

	//oȂݒȂ珈Ȃ
	if( GameConfig.GetBloodFlag() == false ){ return false; }

	float x, y, z;
	float move_x, move_y, move_z;
	float vx, vy, vz, dist, dist2;

	//GtFNg̃p[^[擾
	in_effect->GetPosData(&x, &y, &z, NULL);
	in_effect->GetMove(&move_x, &move_y, &move_z);

	//GtFNgړȂȂ珈Ȃ
	if( (move_x == 0.0f)&&(move_y == 0.0f)&&(move_z == 0.0f) ){ return false; }

	//xNg߂
	dist = sqrtf(move_x*move_x + move_y*move_y + move_z*move_z);
	vx = move_x / dist;
	vy = move_y / dist;
	vz = move_z / dist;

	//C̓蔻
	if( CollD->CheckALLBlockIntersectRay(x, y, z, vx, vy, vz, id, face, &dist2, dist) == false ){
		return false;
	}

	//}bv킸ɕ
	dist2 -= 1.0f;

	//tW߂
	*pos_x = x + vx*dist2;
	*pos_y = y + vy*dist2;
	*pos_z = z + vz*dist2;

	return true;
}

//! @brief E
//! @param in_human Ώۂ̐lIuWFNg
//! @param in_weapon Ώۂ̕IuWFNg
//! @attention l̎ނ ]r ̏ꍇÅ֐͎s܂B
void ObjectManager::PickupWeapon(human *in_human, weapon *in_weapon)
{
	if( in_human == NULL ){ return; }
	if( in_weapon == NULL ){ return; }

	//ȐlȂΏȂ
	if( in_human->GetEnableFlag() == false ){ return; }
	if( in_human->GetHP() <= 0 ){ return; }

	//l̎ނ ]r Ȃ΁As
	int paramid;
	HumanParameter Paraminfo;
	in_human->GetParamData(&paramid, NULL, NULL, NULL);
	GameParamInfo->GetHuman(paramid, &Paraminfo);
	if( Paraminfo.type == 2 ){ return; }

	//Ă镐ŁANgĂȂȂ΁`
	if( (in_weapon->GetEnableFlag() == true)&&(in_weapon->GetUsingFlag() == false) ){
		float human_x, human_y, human_z;
		float weapon_x, weapon_y, weapon_z;
		float x, z;
		float r;

		//lƕ̍W擾
		in_human->GetPosData(&human_x, &human_y, &human_z, NULL);
		in_weapon->GetPosData(&weapon_x, &weapon_y, &weapon_z, NULL);

		//͈͓Ȃ
		if( ((human_y + HUMAN_PICKUPWEAPON_L) <= weapon_y)&&(weapon_y <= (human_y + HUMAN_PICKUPWEAPON_H)) ){
			//vZ
			x = human_x - weapon_x;
			z = human_z - weapon_z;
			r = x*x + z*z;
			//͈͓Ȃ΁AE킹B
			if( r < (HUMAN_PICKUPWEAPON_R * HUMAN_PICKUPWEAPON_R) ){
				in_human->PickupWeapon(in_weapon);
			}
		}
	}
}

//! @brief IuWFNg
void ObjectManager::CleanupPointDataToObject()
{
	for(int i=0; i<MAX_HUMAN; i++){
		HumanIndex[i].SetEnableFlag(false);
	}

	for(int i=0; i<MAX_WEAPON; i++){
		WeaponIndex[i].SetEnableFlag(false);
	}

	for(int i=0; i<MAX_SMALLOBJECT; i++){
		SmallObjectIndex[i].SetEnableFlag(false);
	}

	for(int i=0; i<MAX_BULLET; i++){
		BulletIndex[i].SetEnableFlag(false);
	}

	for(int i=0; i<MAX_GRENADE; i++){
		GrenadeIndex[i].SetEnableFlag(false);
	}

	for(int i=0; i<MAX_EFFECT; i++){
		EffectIndex[i].SetEnableFlag(false);
	}


	if( Resource != NULL ){
		Resource->CleanupHumanTexture();
	}
}

//! @brief |Cgf[^ɃIuWFNgzu
void ObjectManager::LoadPointData()
{
	AddCollisionFlag = false;
	Player_HumanID = 0;

	//FFimjLtOݒ
	for(int i=0; i<MAX_HUMAN; i++){
		FriendlyFire[i] = false;
	}

	//l̃eNX`tbVtO
	for(int i=0; i<MAX_HUMAN; i++){
		Human_FrameTextureRefresh[i] = false;
	}

	/*
	//l|CgT
	for(int i=0; i<PointData->GetTotaldatas(); i++){
		pointdata data;
		PointData->Getdata(&data, i);

		if( data.p1 == 4 ){
			//l̃eNX`o^
			Resource->AddHumanTexture(data.p2);
		}
	}
	*/

	//lEET
	for(int i=0; i<PointData->GetTotaldatas(); i++){
		pointdata data;
		PointData->Getdata(&data, i);

		//lȂ
		if( (data.p1 == 1)||(data.p1 == 6) ){
			pointdata humaninfodata;

			//l|CgT
			if( PointData->SearchPointdata(&humaninfodata, 0x01 + 0x08, 4, 0, 0, data.p2, 0) == 0 ){
				//continue;

				//l|CgȂAƂ肠u  AvƂĒǉBioO̍Čj

				HumanParameter HumanParam;
				int Weapon[TOTAL_HAVEWEAPON];
				GameParamInfo->GetHuman(0, &HumanParam);
				for(int j=0; j<TOTAL_HAVEWEAPON; j++){
					Weapon[j] = HumanParam.Weapon[j];
				}

				//l̃eNX`o^
				Resource->AddHumanTexture(0);

				//lǉ
				if( data.p1 == 6 ){
					Weapon[1] = ID_WEAPON_NONE;
				}

				//vC[Ȃ΁AԍL^
				if( (data.p4 == 0)&&(Player_HumanID == 0) ){
					Player_HumanID = AddHumanIndex(data.x, data.y, data.z, data.r, 0, 0, Weapon, data.p4, true);
				}
				else{
					AddHumanIndex(data.x, data.y, data.z, data.r, 0, 0, Weapon, data.p4, true);
				}
			}
			else{
				//l̃eNX`o^
				Resource->AddHumanTexture(humaninfodata.p2);

				//lƂĒǉ
				AddHumanIndex(data, humaninfodata, true);
			}
		}

		//Ȃ
		if( (data.p1 == 2)||(data.p1 == 7) ){
			AddWeaponIndex(data);
		}

		//Ȃ
		if( data.p1 == 5 ){
			AddSmallObjectIndex(data);
		}
	}

	//O֌W̏
	ObjectLog->ClearLog();
	ObjectLog->InfoLog("Set object...");
}

//! @brief \[X̉
//! @todo SẴGtFNgB
void ObjectManager::Recovery()
{
	//l̃fԍ擾
	int upmodel[TOTAL_UPMODE];
	int armmodel[TOTAL_ARMMODE];
	int legmodel;
	int walkmodel[TOTAL_WALKMODE];
	int runmodel[TOTAL_RUNMODE];
	Resource->GetHumanModel(upmodel, armmodel, &legmodel, walkmodel, runmodel);

	//l
	int HumanID;
	int GetHumanFlag;
	HumanParameter HumanParam;
	for(int i=0; i<MAX_HUMAN; i++){
		if( HumanIndex[i].GetEnableFlag() == true ){
			HumanIndex[i].GetParamData(&HumanID, NULL, NULL, NULL);

			GetHumanFlag = GameParamInfo->GetHuman(HumanID, &HumanParam);

#ifndef ENABLE_BUG_HUMANWEAPON
			if( GetHumanFlag != 0 ){
				HumanIndex[i].SetEnableFlag(false);
				continue;
			}
#endif

			if( GetHumanFlag == 0 ){
				//l̃eNX`o^
				Resource->AddHumanTexture(HumanID);

				int id = Resource->GetHumanTexture(HumanID);
				if( id == -1 ){
					//eNX`擾łȂ΁AtbVtOLɂĖt[擾
					Human_FrameTextureRefresh[i] = true;
				}

				HumanIndex[i].SetTexture(id);
				HumanIndex[i].SetModel(upmodel[ HumanParam.model ], armmodel, legmodel, walkmodel, runmodel);
			}
			else{
#ifdef ENABLE_BUG_HUMANWEAPON
				HumanIndex[i].SetTexture(d3dg->GetMapTextureID(0));
				HumanIndex[i].SetModel(upmodel[0], armmodel, legmodel, walkmodel, runmodel);
#endif
			}
		}
	}

	//
	int WeaponID;
	int Weaponmodel, Weapontexture;
	for(int i=0; i<MAX_WEAPON; i++){
		if( WeaponIndex[i].GetEnableFlag() == true ){
			//ݒl擾
			WeaponParameter WeaponParam;
			WeaponIndex[i].GetParamData(&WeaponID, NULL, NULL);
			if( Resource->GetWeaponModelTexture(WeaponID, &Weaponmodel, &Weapontexture) == 1 ){ continue; }
			if( GameParamInfo->GetWeapon(WeaponID, &WeaponParam) == 1 ){ continue; }

			//Kp
			WeaponIndex[i].SetModel(Weaponmodel, WeaponParam.size);
			WeaponIndex[i].SetTexture(Weapontexture);
		}
	}

	//
	int SmallObjectID;
	int SmallObjectmodel, SmallObjecttexture;
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		if( SmallObjectIndex[i].GetEnableFlag() == true ){
			//ݒl擾
			SmallObjectIndex[i].GetParamData(&SmallObjectID, NULL);
			if( Resource->GetSmallObjectModelTexture(SmallObjectID, &SmallObjectmodel, &SmallObjecttexture) == 1 ){ continue; }

			//Kp
			SmallObjectIndex[i].SetModel(SmallObjectmodel, SMALLOBJECT_SCALE);
			SmallObjectIndex[i].SetTexture(SmallObjecttexture);
		}
	}

	int bulletmodel, bullettexture;
	BulletParameter data;

	//eeKp
	if( Resource->GetBulletModelTexture(0, &bulletmodel, &bullettexture) == 0 ){
		if( GameParamInfo->GetBullet(0, &data) == 0 ){
			for(int i=0; i<MAX_BULLET; i++){
				BulletIndex[i].SetModel(bulletmodel, data.size);
				BulletIndex[i].SetTexture(bullettexture);
			}
		}
	}

	//֒eKp
	if( Resource->GetBulletModelTexture(1, &bulletmodel, &bullettexture) == 0 ){
		if( GameParamInfo->GetBullet(1, &data) == 0 ){
			for(int i=0; i<MAX_GRENADE; i++){
				GrenadeIndex[i].SetModel(bulletmodel, data.size);
				GrenadeIndex[i].SetTexture(bullettexture);
			}
		}
	}

	//SẴGtFNg𖳌
	for(int i=0; i<MAX_EFFECT; i++){
		if( EffectIndex[i].GetEnableFlag() == true ){
			EffectIndex[i].SetEnableFlag(false);
		}
	}

	//O֌W̏
	ObjectLog->InfoLog("Recovery object...");
}

//! @brief ǉ̓蔻tOݒ
//! @param flag tO
void ObjectManager::SetAddCollisionFlag(bool flag)
{
	AddCollisionFlag = flag;
}

//! @brief FFimjLtO擾
//! @param id f[^ԍ
//! @return tO
bool ObjectManager::GetFriendlyFireFlag(int id)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return false; }
	return FriendlyFire[id];
}

//! @brief FFimjLtOݒ
//! @param id f[^ԍ
//! @param flag tO
void ObjectManager::SetFriendlyFireFlag(int id, bool flag)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return; }
	FriendlyFire[id] = flag;
}

//! @brief vC[ԍ擾
//! @return vC[̃f[^ԍ
int ObjectManager::GetPlayerID()
{
	return Player_HumanID;
}

//! @brief vC[ԍݒ
//! @param id vC[̃f[^ԍ
void ObjectManager::SetPlayerID(int id)
{
	Player_HumanID = id;
}

//! @brief w肵f[^ԍhumanNX擾
//! @param id f[^ԍ
//! @return lIuWFNg̃|C^@iȃf[^ԍ NULLj
human* ObjectManager::GetHumanObject(int id)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return NULL; }
	return &(HumanIndex[id]);
}

//! @brief vC[humanNX擾
//! @return lIuWFNgivC[j̃|C^
human* ObjectManager::GetPlayerHumanObject()
{
	return GetHumanObject(Player_HumanID);
}

//! @brief w肵human|C^̃f[^ԍ擾
//! @param object lIuWFNg̃|C^
//! @return f[^ԍ@iG[F-1j
int ObjectManager::GetHumanObjectID(human* object)
{
	if( object == NULL ){ return -1; }

	for(int i=0; i<MAX_HUMAN; i++){
		if( &(HumanIndex[i]) == object ){
			return i;
		}
	}

	return -1;
}

//! @brief w肵f[^ԍweaponNX擾
//! @param id f[^ԍ
//! @return IuWFNg̃|C^@iȃf[^ԍ NULLj
weapon* ObjectManager::GetWeaponObject(int id)
{
	if( (id < 0)||(MAX_WEAPON <= id) ){ return NULL; }
	return &(WeaponIndex[id]);
}

//! @brief w肵f[^ԍsmallobjectNX擾
//! @param id f[^ԍ
//! @return IuWFNg̃|C^@iȃf[^ԍ NULLj
smallobject* ObjectManager::GetSmallObject(int id)
{
	if( (id < 0)||(MAX_SMALLOBJECT <= id) ){ return NULL; }
	return &(SmallObjectIndex[id]);
}

//! @brief w肵f[^ԍbulletNX擾
//! @param id f[^ԍ
//! @return eIuWFNg̃|C^@iȃf[^ԍ NULLj
bullet* ObjectManager::GetBulletObject(int id)
{
	if( (id < 0)||(MAX_BULLET <= id) ){ return NULL; }
	return &(BulletIndex[id]);
}

//! @brief w肵bullet|C^̃f[^ԍ擾
//! @param object eIuWFNg̃|C^
//! @return f[^ԍ@iG[F-1j
int ObjectManager::GetBulletObjectID(bullet* object)
{
	if( object == NULL ){ return -1; }

	for(int i=0; i<MAX_BULLET; i++){
		if( &(BulletIndex[i]) == object ){
			return i;
		}
	}

	return -1;
}

//! @brief gpĂȂbulletNX擾
//! @return ݖgp̒eIuWFNg̃|C^@is NULLj
bullet* ObjectManager::GetNewBulletObject()
{
	for(int i=0; i<MAX_BULLET; i++){
		if( BulletIndex[i].GetEnableFlag() == false ){
			return &(BulletIndex[i]);
		}
	}
	return NULL;
}

//! @brief gpĂȂgrenadeNX擾
//! @return ݖgp̎֒eIuWFNg̃|C^@is NULLj
grenade* ObjectManager::GetNewGrenadeObject()
{
	for(int i=0; i<MAX_GRENADE; i++){
		if( GrenadeIndex[i].GetEnableFlag() == false ){
			return &(GrenadeIndex[i]);
		}
	}
	return NULL;
}

//! @brief l
//! @param p4 Ώۂ̔Fԍ
//! @return YhumanNX̃|C^@iȂꍇNULLj
//! @attention YꍇAŏɊYf[^Ԃ܂B
human* ObjectManager::SearchHuman(signed short int p4)
{
	signed short int humanp4;

	for(int i=0; i<MAX_HUMAN; i++){
		//gĂȂlȂΏȂ
		if( HumanIndex[i].GetEnableFlag() == false ){ continue; }

		//4p[^擾
		HumanIndex[i].GetParamData(NULL, NULL, &humanp4, NULL);

		//w肳ꂽp4ƈvΕԂ
		if( humanp4 == p4 ){
			return &(HumanIndex[i]);
		}
	}
	return NULL;
}

//! @brief 
//! @param p4 Ώۂ̔Fԍ
//! @return YsmallobjectNX̃|C^@iȂꍇNULLj
//! @attention YꍇAŏɊYf[^Ԃ܂B
smallobject* ObjectManager::SearchSmallobject(signed short int p4)
{
	signed short int smallobjectp4;

	for(int i=0; i<MAX_SMALLOBJECT; i++){
		//gĂȂlȂΏȂ
		//@yjς݂̃IuWFNg肷邽߁Az
		//if( SmallObjectIndex[i].GetEnableFlag() == false ){ continue; }

		//4p[^擾
		SmallObjectIndex[i].GetParamData(NULL, &smallobjectp4);

		//w肳ꂽp4ƈvΕԂ
		if( smallobjectp4 == p4 ){
			return &(SmallObjectIndex[i]);
		}
	}
	return NULL;
}

//! @brief LȃIuWFNg擾
//! @param HumanCnt lIuWFNg̗L󂯎|C^@iNULLj
//! @param WeaponCnt IuWFNg̗L󂯎|C^@iNULLj
//! @param SmallobjectCnt IuWFNg̗L󂯎|C^@iNULLj
//! @param BulletCnt eIuWFNg̗L󂯎|C^@iNULLj
//! @param GrenadeCnt ֒eIuWFNg̗L󂯎|C^@iNULLj
//! @param EffectCnt GtFNgIuWFNg̗L󂯎|C^@iNULLj
void ObjectManager::GetTotalObjects(int *HumanCnt, int *WeaponCnt, int *SmallobjectCnt, int *BulletCnt, int *GrenadeCnt, int *EffectCnt)
{
	if( HumanCnt != NULL ){
		*HumanCnt = 0;
		for(int i=0; i<MAX_HUMAN; i++){
			if( HumanIndex[i].GetEnableFlag() == true ){ *HumanCnt += 1; }
		}
	}

	if( WeaponCnt != NULL ){
		*WeaponCnt = 0;
		for(int i=0; i<MAX_WEAPON; i++){
			if( WeaponIndex[i].GetEnableFlag() == true ){ *WeaponCnt += 1; }
		}
	}

	if( SmallobjectCnt != NULL ){
		*SmallobjectCnt = 0;
		for(int i=0; i<MAX_SMALLOBJECT; i++){
			if( SmallObjectIndex[i].GetEnableFlag() == true ){ *SmallobjectCnt += 1; }
		}
	}

	if( BulletCnt != NULL ){
		*BulletCnt = 0;
		for(int i=0; i<MAX_BULLET; i++){
			if( BulletIndex[i].GetEnableFlag() == true ){ *BulletCnt += 1; }
		}
	}

	if( GrenadeCnt != NULL ){
		*GrenadeCnt = 0;
		for(int i=0; i<MAX_GRENADE; i++){
			if( GrenadeIndex[i].GetEnableFlag() == true ){ *GrenadeCnt += 1; }
		}
	}

	if( EffectCnt != NULL ){
		*EffectCnt = 0;
		for(int i=0; i<MAX_EFFECT; i++){
			if( EffectIndex[i].GetEnableFlag() == true ){ *EffectCnt += 1; }
		}
	}
}

//! @brief Oiijs
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveForward(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	//IuWFNgɃtOݒ
	HumanIndex[human_id].SetMoveForward();
}

//! @brief ނs
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveBack(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	//IuWFNgɃtOݒ
	HumanIndex[human_id].SetMoveBack();
}

//! @brief s
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveLeft(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	//IuWFNgɃtOݒ
	HumanIndex[human_id].SetMoveLeft();
}

//! @brief Es
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveRight(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	//IuWFNgɃtOݒ
	HumanIndex[human_id].SetMoveRight();
}

//! @brief s
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveWalk(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	//IuWFNgɃtOݒ
	HumanIndex[human_id].SetMoveWalk();
}

//! @brief Wv
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::MoveJump(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	HumanIndex[human_id].Jump();
}

//! @brief C
//! @param human_id Clԍ
//! @return ʏeˁF1@֒eˁF2@sF0
int ObjectManager::ShotWeapon(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return 0; }

	human *MyHuman = &(HumanIndex[human_id]);

	float pos_x, pos_y, pos_z;
	int teamid;
	float rotation_x, armrotation_y;
	int weapon_paramid;
	int GunsightErrorRange;
	WeaponParameter ParamData;
	bool playerflag;
	bool grenadeflag;

	//l̍WƊpx擾
	MyHuman->GetPosData(&pos_x, &pos_y, &pos_z, NULL);
	MyHuman->GetParamData(NULL, NULL, NULL, &teamid);
	MyHuman->GetRxRy(&rotation_x, &armrotation_y);

	//Ώێ҂vC[g
	if( MyHuman == &(HumanIndex[Player_HumanID]) ){
		playerflag = true;
	}
	else{
		playerflag = false;
	}

	//e̔˂v
	if( MyHuman->ShotWeapon(&weapon_paramid, &GunsightErrorRange) == false ){ return 0; }

	//̏擾
	if( GameParamInfo->GetWeapon(weapon_paramid, &ParamData) != 0 ){ return 0; }

	//֒e
	if( weapon_paramid == ID_WEAPON_GRENADE ){ grenadeflag = true; }
	else{ grenadeflag = false; }

	//덷͈̔͂vZ
	int ErrorRange;
	ErrorRange = GunsightErrorRange;
	if( ErrorRange < ParamData.ErrorRangeMIN ){ ErrorRange = ParamData.ErrorRangeMIN; }

	//ˊpxi덷[̃x[Xj
	float rx, ry;
	rx = rotation_x*-1 + (float)M_PI/2;
	ry = armrotation_y;

	//덷
	if( ErrorRange != 0 ){
		int ErrorRange_x, ErrorRange_y;

		ErrorRange_x = GetRand(ErrorRange*2+1) - ErrorRange;
		ErrorRange_y = GetRand(ErrorRange*2+1) - ErrorRange;

		//ˌ덷@pxɉZ
		rx += ErrorRange_x * DegreeToRadian(0.15f);
		ry += ErrorRange_y * DegreeToRadian(0.15f);
	}

	//֒ełȂ
	if( grenadeflag == false ){
		class bullet* newbullet;

		//VbgKȂǔ˂eiybgj̐JԂ
		for(int i=0; i<ParamData.pellet; i++){
			int attacks;
			float ontargetcnt;
			float rx2, ry2;

			//VbgKȂǔ˂eiybgj
			if( ParamData.pellet > 1 ){
				//1̒e̍U͂Ɩ̃JEgZo
				//@@Se킹āA2{ɂȂ悤ɂB
				attacks = (int)( (float)ParamData.attacks / ((float)ParamData.pellet/2) );
				ontargetcnt = 1.0f / ((float)ParamData.pellet/2);

				//Ɍ덷𑝂₵ĊgU
				int len;
				float a = DegreeToRadian(10) * GetRand(36);
				len = GetRand(5)*2+5;
				rx2 = rx + cosf(a)*len * DegreeToRadian(0.15f);
				ry2 = ry + sinf(a)*len * DegreeToRadian(0.15f);
			}
			else{
				//̂܂܍U͂Ɩ̃JEg𔽉f
				attacks = ParamData.attacks;
				ontargetcnt = 1.0f;

				//ˌ덷͂̂܂
				rx2 = rx;
				ry2 = ry;
			}

			//˂関gp̃IuWFNg擾
			newbullet = GetNewBulletObject();
			if( newbullet == NULL ){ return 0; }

			//ee𔭎
			newbullet->SetPosData(pos_x, pos_y + WEAPONSHOT_HEIGHT, pos_z, rx2, ry2);
			newbullet->SetParamData(attacks, ParamData.penetration, (int)((float)ParamData.speed * BULLET_SPEEDSCALE), teamid, human_id, ontargetcnt, true);
			newbullet->SetEnableFlag(true);

			//ΐlpXg
			BulletObj_HumanIndex[ GetBulletObjectID(newbullet) ].Init();
		}

		//CtOݒ
		Human_ShotFlag[human_id] = true;
	}
	else{
		class grenade* newgrenade;

		//˂関gp̃IuWFNg擾
		newgrenade = GetNewGrenadeObject();
		if( newgrenade == NULL ){ return 0; }

		//֒e
		newgrenade->SetPosData(pos_x, pos_y + WEAPONSHOT_HEIGHT, pos_z, rx, ry);
		newgrenade->SetParamData(8.0f, teamid, human_id, 1.0f, true);
		newgrenade->SetEnableFlag(true);
	}

	if( ParamData.soundvolume > 0 ){
		//eĐ
		GameSound->ShotWeapon(pos_x, pos_y + WEAPONSHOT_HEIGHT, pos_z, weapon_paramid, teamid, playerflag);
	}

	if( grenadeflag == true ){
		return 2;
	}
	return 1;
}

//! @brief }YtbVݒ
//! @param humanid l̔ԍ
//! @attention ̊֐̌Ăяo^C~OƁAeɑ΂ă}YtbV܂B
void ObjectManager::ShotWeaponEffect(int humanid)
{
	//l͈̔͂`FbN
	if( (humanid < 0)||(MAX_HUMAN <= humanid) ){ return; }

	float pos_x, pos_y, pos_z;
	float rotation_x, armrotation_y;
	int weapon_paramid;
	WeaponParameter ParamData;
	float x, y, z;
	float flashsize, smokesize1, smokesize2, yakkyousize;
	float rx, emx, emy, emz;
	float mx, my, mz;

	//l̍WƊpx擾
	HumanIndex[humanid].GetPosData(&pos_x, &pos_y, &pos_z, NULL);
	HumanIndex[humanid].GetRxRy(&rotation_x, &armrotation_y);

	//̏擾
	weapon_paramid = HumanIndex[humanid].GetMainWeaponTypeNO();
	if( GameParamInfo->GetWeapon(weapon_paramid, &ParamData) != 0 ){ return; }

	//}YtbVƉ̃TCY
	flashsize = 0.06f * ParamData.attacks;
	smokesize1 = 0.04f * ParamData.attacks;
	smokesize2 = 0.032f * ParamData.attacks;
	yakkyousize = 0.012f * ParamData.attacks;
	if( ParamData.silencer == true ){
		flashsize = 1.2f;
	}

	//̈ړxNg
	rx = rotation_x*-1 + (float)M_PI/2;
	emx = cosf(rx)*cosf(armrotation_y);
	emy = sinf(armrotation_y);
	emz = sinf(rx)*cosf(armrotation_y);

	//䰂̈ړʂ
	mx = cosf(rx - (float)M_PI/2) * ParamData.yakkyou_sx * 0.12f;
	my = (ParamData.yakkyou_sy + (GetRand(3)-1)) * 0.12f;
	mz = sinf(rx - (float)M_PI/2) * ParamData.yakkyou_sx * 0.12f;

	//sŃGtFNgWvZ
	d3dg->SetWorldTransformHumanWeapon(pos_x, pos_y + 16.0f, pos_z, ParamData.flashx/10, ParamData.flashy/10, ParamData.flashz/10, rotation_x, armrotation_y*-1, 1.0f);
	d3dg->GetWorldTransformPos(&x, &y, &z);
	d3dg->ResetWorldTransform();

	//}YtbV`
	AddEffect(x, y, z, 0.0f, 0.0f, 0.0f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, flashsize, 0.0f, Resource->GetEffectMflashTexture(), 0, 0, 0.63f, 0.0f, EFFECT_NORMAL, 1);

	//sŃGtFNgWvZ
	d3dg->SetWorldTransformHumanWeapon(pos_x, pos_y + 16.0f, pos_z, ParamData.flashx/10, ParamData.flashy/10, ParamData.flashz/10 - 0.1f, rotation_x, armrotation_y*-1, 1.0f);
	d3dg->GetWorldTransformPos(&x, &y, &z);
	d3dg->ResetWorldTransform();

	//GtFNgij̕`
	AddEffect(x, y, z, emx*0.15f, emy*0.15f + 0.02f, emz*0.15f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, smokesize1, 0.3f, Resource->GetEffectSmokeTexture(), 0, 0, 0.47f, -0.047f, EFFECT_NORMAL, 12);
	AddEffect(x, y, z, emx*0.45f, emy*0.45f, emz*0.45f, 0.0f, DegreeToRadian(10)*GetRand(36), 0.0f, smokesize2, 0.36f, Resource->GetEffectSmokeTexture(), 0, 0, 0.47f, -0.055f, EFFECT_NORMAL, 10);

	//sŃGtFNgWvZ
	d3dg->SetWorldTransformHumanWeapon(pos_x, pos_y + 16.0f, pos_z, ParamData.yakkyou_px/10, ParamData.yakkyou_py/10, ParamData.yakkyou_pz/10, rotation_x, armrotation_y*-1, 1.0f);
	d3dg->GetWorldTransformPos(&x, &y, &z);
	d3dg->ResetWorldTransform();

	//䰕`
	AddEffect(x, y, z, mx, my, mz, -0.15f, DegreeToRadian(10)*GetRand(36), DegreeToRadian(GetRand(61)-30), yakkyousize, 0.0f, Resource->GetEffectYakkyouTexture(), 0, 0, 1.0f, 0.0f, EFFECT_NORMAL, (int)(GAMEFPS*0.75f));
}

//! @brief [h
//! @param human_id Ώۂ̐lԍ
//! @return Ftrue@sFfalse
bool ObjectManager::ReloadWeapon(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return false; }

	//ȐlȂΏȂ
	if( HumanIndex[human_id].GetEnableFlag() == false ){ return false; }
	if( HumanIndex[human_id].GetHP() <= 0 ){ return false; }

	//[hs
	if( HumanIndex[human_id].ReloadWeapon() == true ){
		float x, y, z;
		int id;

		//l̍Wƃ`[ԍ擾
		HumanIndex[human_id].GetPosData(&x, &y, &z, NULL);
		HumanIndex[human_id].GetParamData(NULL, NULL, NULL, &id);
		y += 16.0f;

		//zu
		GameSound->ReloadWeapon(x, y, z, id);

		return true;
	}

	return false;
}

//! @brief ؂ւiւj
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
//! @param id ւ镐@i-1 Ŏ̕j
void ObjectManager::ChangeHaveWeapon(int human_id, int id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	HumanIndex[human_id].ChangeHaveWeapon(id);
}

//! @brief ̎ސ؂ւ
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
//! @return F0@sF1
int ObjectManager::ChangeWeaponID(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return 1; }

	//̎ނ؂ւ
	if( HumanIndex[human_id].ChangeWeaponID() == 0 ){
		int selectweapon;
		weapon *weapon[TOTAL_HAVEWEAPON];
		int WeaponID;
		int model, texture;
		float WeaponSize;

		//݂̕擾
		HumanIndex[human_id].GetWeapon(&selectweapon, weapon, NULL, NULL);
		weapon[selectweapon]->GetParamData(&WeaponID, NULL, NULL);

		//fƃeNX`擾
		if( Resource->GetWeaponModelTexture(WeaponID, &model, &texture) == 1 ){
			return 1;
		}

		//̃TCY擾
		WeaponParameter ParamData;
		if( GameParamInfo->GetWeapon(WeaponID, &ParamData) != 0 ){ return 1; }
		WeaponSize = ParamData.size;

		//IuWFNg̃fEeNX`ύX
		weapon[selectweapon]->SetModel(model, WeaponSize);
		weapon[selectweapon]->SetTexture(texture);

		return 0;
	}
	return 1;
}

//! @brief ̂Ă
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
//! @return Ftrue@sFfalse
bool ObjectManager::DumpWeapon(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return false; }

	return HumanIndex[human_id].DumpWeapon();
}

//! @brief XR[v[h؂ւ
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
void ObjectManager::ChangeScopeMode(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return; }

	if( HumanIndex[human_id].GetScopeMode() == 0 ){	//XR[vgpĂȂ΁AXR[vݒ
		HumanIndex[human_id].SetEnableScope();
	}
	else{											//gpȂA
		HumanIndex[human_id].SetDisableScope();
	}
}

//! @brief ZEĂ镐̒eǉ
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
//! @return Ftrue@sFfalse
bool ObjectManager::CheatAddBullet(int human_id)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return false; }

	int selectweapon;
	weapon *weapon[TOTAL_HAVEWEAPON];
	int id_param, lnbs, nbs;
	WeaponParameter ParamData;

	//Ă镐擾
	for(int i=0; i<TOTAL_HAVEWEAPON; i++){
		weapon[i] = NULL;
	}
	HumanIndex[human_id].GetWeapon(&selectweapon, weapon, NULL, NULL);

	//̕Ă
	if( weapon[selectweapon] != NULL ){
		//̎ނƒeA̐ݒl擾
		weapon[selectweapon]->GetParamData(&id_param, &lnbs, &nbs);
		if( GameParamInfo->GetWeapon(id_param, &ParamData) == 0 ){
			//őeZAKp
			nbs += ParamData.nbsmax;
			weapon[selectweapon]->ResetWeaponParam(Resource, id_param, lnbs, nbs);
			return true;
		}
	}

	return false;
}

//! @brief ZEĂ镐ύX
//! @param human_id l̔ԍi0`MAX_HUMAN-1j
//! @param new_weaponID Vɐݒ肷镐̎ޔԍ
//! @return Ftrue@sFfalse
bool ObjectManager::CheatNewWeapon(int human_id, int new_weaponID)
{
	//l͈̔͂`FbN
	if( (human_id < 0)||(MAX_HUMAN <= human_id) ){ return false; }
	if( (new_weaponID < 0)||(TOTAL_PARAMETERINFO_WEAPON <= new_weaponID) ){ return false; }

	human *myHuman = &(HumanIndex[human_id]);

	int selectweapon;
	weapon *weapon[TOTAL_HAVEWEAPON];
	int lnbslist[TOTAL_HAVEWEAPON];
	int nbslist[TOTAL_HAVEWEAPON];
	int lnbs, nbs;

	//Ă镐擾
	for(int i=0; i<TOTAL_HAVEWEAPON; i++){
		weapon[i] = NULL;
		lnbslist[i] = 0;
		nbslist[i] = 0;
	}
	myHuman->GetWeapon(&selectweapon, weapon, lnbslist, nbslist);

	//ݕLĂ炸AVɎ镐ȂȂ
	if( (weapon[selectweapon] == NULL)&&(new_weaponID == ID_WEAPON_NONE) ){
		return false;		//̂Ӗ
	}

	if( weapon[selectweapon] == NULL ){		//LĂȂ
		int dataid = -1;

		//Vzu
		dataid = AddVisualWeaponIndex(new_weaponID, false);

		//킪zuł
		if( dataid != -1 ){
			//ݒKpiɒej
			weapon[selectweapon] = &(WeaponIndex[dataid]);
			weapon[selectweapon]->ResetWeaponParam(Resource, new_weaponID, lnbslist[selectweapon], nbslist[selectweapon]);

			//̒e͏
			lnbslist[selectweapon] = 0;
			nbslist[selectweapon] = 0;

			//
			myHuman->SetWeapon(weapon, lnbslist, nbslist);
			return true;
		}
	}
	else{									//LĂ
		//ݒ擾ieݒp߁j
		weapon[selectweapon]->GetParamData(NULL, &lnbs, &nbs);

		if( new_weaponID == ID_WEAPON_NONE ){			//Ȃ
			//͍̕폜
			weapon[selectweapon]->SetEnableFlag(false);

			//폜Ae𔽉f
			weapon[selectweapon] = NULL;
			lnbslist[selectweapon] = lnbs;
			nbslist[selectweapon] = nbs;
			myHuman->SetWeapon(weapon, lnbslist, nbslist);

			//XR[v[h
			myHuman->SetDisableScope();
		}
		else{											//ύXȂ
			//ݒKp
			weapon[selectweapon]->ResetWeaponParam(Resource, new_weaponID, lnbs, nbs);
		}
		return true;
	}

	return false;
}

//! @brief ]r̍U󂯂邩
//! @param MyHuman UlIuWFNgi]rj̃|C^
//! @param EnemyHuman U󂯂lIuWFNg̃|C^
//! @return Ftrue@sFfalse
//! @warning MyHuman ̓]rȊOwł܂B<b>]rǂ͔肵Ȃ</b>߁A]rł邩\ߊmFĂKv܂B
//! @attention ݂̂{܂B̔肪 HitZombieAttack()֐ ĂяoĂB 
bool ObjectManager::CheckZombieAttack(human* MyHuman, human* EnemyHuman)
{
	if( MyHuman == NULL ){ return false; }
	if( EnemyHuman == NULL ){ return false; }

	//gpĂȂASĂΏȂB
	if( MyHuman->GetEnableFlag() == false ){ return false; }
	if( MyHuman->GetHP() <= 0 ){ return false; }
	if( EnemyHuman->GetEnableFlag() == false ){ return false; }
	if( EnemyHuman->GetDeadFlag() == true ){ return false; }

	float mx, my, mz, mrx, tx, ty, tz;
	int mteam, tteam;
	float AttackPoint_x, AttackPoint_y, AttackPoint_z;
	float ax, az;

	MyHuman->GetPosData(&mx, &my, &mz, &mrx);
	MyHuman->GetParamData(NULL, NULL, NULL, &mteam);
	my += VIEW_HEIGHT;
	EnemyHuman->GetPosData(&tx, &ty, &tz, NULL);
	EnemyHuman->GetParamData(NULL, NULL, NULL, &tteam);
	ty += VIEW_HEIGHT;

	//ȂΏȂ
	if( mteam == tteam ){ return false; }

	//U|Cgir̐[j߂
	AttackPoint_x = mx + cosf(mrx*-1 + (float)M_PI/2) * 2.0f;
	AttackPoint_y = my + VIEW_HEIGHT - 0.5f;
	AttackPoint_z = mz + sinf(mrx*-1 + (float)M_PI/2) * 2.0f;
	ax = AttackPoint_x - tx;
	az = AttackPoint_z - tz;

	//GiUΏہjU|CgɐGĂ΁AĂ
	if( (ax*ax + az*az) < 3.3f*3.3f ){
		if( (AttackPoint_y >= ty)&&(AttackPoint_y <= (ty + HUMAN_HEIGHT)) ){
			return true;
		}
	}

	return false;
}

//! @brief ]r̍U󂯂
//! @param MyHuman UlIuWFNgi]rj̃|C^
//! @param EnemyHuman U󂯂lIuWFNg̃|C^
void ObjectManager::HitZombieAttack(human* MyHuman, human* EnemyHuman)
{
	if( MyHuman == NULL ){ return; }
	if( EnemyHuman == NULL ){ return; }

	//gpĂȂASĂΏȂB
	if( EnemyHuman->GetEnableFlag() == false ){ return; }
	if( EnemyHuman->GetDeadFlag() == true ){ return; }

	int MyHuman_dataID, MyHuman_TeamID;
	int EnemyHuman_dataID, EnemyHuman_TeamID;
	float mx, my, mz;
	float tx, ty, tz;
	float arx;
	int paramid;
	HumanParameter Paraminfo;
	bool NotRobot;
	int hp_old;

	//]rƍU󂯂鑤`[ԍ擾
	MyHuman->GetParamData(NULL, &MyHuman_dataID, NULL, &MyHuman_TeamID);
	EnemyHuman->GetParamData(NULL, &EnemyHuman_dataID, NULL, &EnemyHuman_TeamID);

	MyHuman->GetPosData(&mx, &my, &mz, NULL);
	EnemyHuman->GetPosData(&tx, &ty, &tz, NULL);
	ty += VIEW_HEIGHT;

	//GU󂯂px߂
	arx = atan2f(mz - tz, mx - tx);

	//{bgǂ
	EnemyHuman->GetParamData(&paramid, NULL, NULL, NULL);
	GameParamInfo->GetHuman(paramid, &Paraminfo);
	if( Paraminfo.type == 1 ){
		NotRobot = false;
	}
	else{
		NotRobot = true;
	}

	//_[WvZOHP擾
	hp_old = EnemyHuman->GetHP();

	//_[WȂǂvZ
	EnemyHuman->HitZombieAttack();

#ifdef ENABLE_BUG_TEAMID
	bool flag = true;

	//`[ԍA`[ԍ傫ȂAtO
	if( (EnemyHuman_TeamID < 0)&&(MyHuman_TeamID < 0) ){
		if( EnemyHuman_TeamID < MyHuman_TeamID ){
			flag = false;
		}
	}

	if( flag == true ){
		EnemyHuman->SetHitFlag(arx*-1 + (float)M_PI/2);
	}
#else
	EnemyHuman->SetHitFlag(arx*-1 + (float)M_PI/2);
#endif

	//GtFNgij`
	SetHumanBlood(tx, ty, tz, HUMAN_DAMAGE_ZOMBIEU, NotRobot);

	//ʉĐ
	GameSound->HitHuman(tx, ty, tz, 3, MyHuman_TeamID);

	//O֌W̏
	if( (hp_old > 0)&&(EnemyHuman->GetHP() <= 0) ){
		//_[WvZO hp>0 łAvZ hp <= 0 ȂA񎀖SB

		int player_teamid;
		HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);

		ObjectLog->DiedHuman(MyHuman_dataID, EnemyHuman_dataID, MyHuman_TeamID, EnemyHuman_TeamID, player_teamid);
	}
}

//! @brief ҂h
//! @param id l̔ԍi0`MAX_HUMAN-1j
//! @return trueF@falseFs
//! @warning Ԃ̂܂ܑh܂
//! @attention Ȑlԍw肳ꂽꍇAw肵lĂꍇA邢͓lԂɑ΂ĎsꍇÅ֐͎s܂B
bool ObjectManager::HumanResuscitation(int id)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return false; }

	//gpĂȂAĂΏȂB
	if( HumanIndex[id].GetEnableFlag() == false ){ return false; }
	if( HumanIndex[id].GetDeadFlag() == false ){ return false; }

	int id_param, dataid, team;
	signed short int p4;
	HumanIndex[id].GetParamData(&id_param, &dataid, &p4, &team);
	if( (id_param < 0)||( TOTAL_PARAMETERINFO_HUMAN <= id_param) ){ return false; }	//lԂȂ珈Ȃ
	HumanIndex[id].SetParamData(id_param, dataid, p4, team, true);

	//O֌W̏
	int player_teamid;
	HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);
	ObjectLog->ReviveHuman(id, team, player_teamid);

	return true;
}

//! @brief Q[NA[EQ[I[o[̔
//! @return Q[NA[F1@Q[I[o[F2@ȂF0
//! @attention Q[NA\ƃQ[I[o[ɐł́A{XOPSƓlɁuQ[NA[vƔ肳܂B
int ObjectManager::CheckGameOverorComplete()
{
	//F
	//
	//@{́AQ[I[o[ɍsAɃQ[NA[IłB
	//@A{XOPS̓Q[NA[̔D悷邽߁AɃQ[NA[̔sĂ܂B
	//@ĂG̐iHPƂājAG̐iHPj 0 Ȃ΁AQ[NA[Ɣ肵܂B

	//Q[NA[
	int MyTeamid, teamid;
	int TotalEnemyHP = 0;
	HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &MyTeamid);	//vC[̃`[ԍ擾
	for(int i=0; i<MAX_HUMAN; i++){
		//ĂȂΏȂ
		if( HumanIndex[i].GetEnableFlag() == false ){ continue; }

		//ׂΏۂ̃`[ԍ擾
		HumanIndex[i].GetParamData(NULL, NULL, NULL, &teamid);

		//قȂ`[ԍiGjȂ
		if( teamid != MyTeamid ){
			//Ă΁AGHPƂĉZ
			//if( HumanIndex[i].GetDeadFlag() == false ){
			if( HumanIndex[i].GetHP() > 0 ){
				TotalEnemyHP += HumanIndex[i].GetHP();
			}
		}
	}
	if( TotalEnemyHP == 0 ){	//SGHP 0 Ȃ
		return 1;
	}


	//Q[I[o[
	if( HumanIndex[Player_HumanID].GetEnableFlag() == true ){	//ΏۂLȂ΁@iFZɂύX΍j
		if( HumanIndex[Player_HumanID].GetDeadFlag() == true ){		//vC[SĂ
			return 2;
		}
	}

	return 0;
}

//! @brief IuWFNg̏擾
//! @param camera_x JXW
//! @param camera_y JYW
//! @param camera_z JZW
//! @param camera_rx J̉px
//! @param camera_ry J̏cpx
//! @param color F擾|C^
//! @param infostr 擾|C^
//! @return \񂠂Ftrue@\ȂFfalse
bool ObjectManager::GetObjectInfoTag(float camera_x, float camera_y, float camera_z, float camera_rx, float camera_ry, int *color, char *infostr)
{
	if( color == NULL ){ return false; }
	if( infostr == NULL ){ return false; }

	float dist = 50.0f;
	float px, py, pz;
	float rx, ry;
	float r;
	int Player_teamID;

	//
	infostr[0] = '\0';

	//vC[̃`[ԍ擾
	HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &Player_teamID);

	//l
	for(int i=0; i<MAX_HUMAN; i++){
		//vC[gȂ珈Ȃ
		if( i == Player_HumanID ){ continue; }

		if( HumanIndex[i].GetEnableFlag() == true ){
			HumanIndex[i].GetPosData(&px, &py, &pz, NULL);

			//_ɑΏۂ܂ł̊pxZo
			if( CheckTargetAngle(camera_x, camera_y, camera_z, camera_rx, 0.0f, px, py, pz, dist, &rx, NULL, &r) == true ){
				int team;

				//pxAEɓĂ
				if( fabsf(rx) < DegreeToRadian(10) ){
					HumanIndex[i].GetParamData(NULL, NULL, NULL, &team);
					if( team == Player_teamID ){
						*color = d3dg->GetColorCode(0.0f,0.0f,1.0f,1.0f);
						sprintf(infostr, "Human[%d]  HP %d : Friend", i, HumanIndex[i].GetHP());
					}
					else{
						*color = d3dg->GetColorCode(1.0f,0.0f,0.0f,1.0f);
						sprintf(infostr, "Human[%d]  HP %d : Enemy", i, HumanIndex[i].GetHP());
					}
					dist = sqrtf(r);
				}
			}
		}
	}

	//
	for(int i=0; i<MAX_WEAPON; i++){
		if( (WeaponIndex[i].GetEnableFlag() == true)&&(WeaponIndex[i].GetUsingFlag() == false) ){
			int lnbs, nbs;

			WeaponIndex[i].GetPosData(&px, &py, &pz, NULL);
			WeaponIndex[i].GetParamData(NULL, &lnbs, &nbs);

			//_ɑΏۂ܂ł̊pxZo
			if( CheckTargetAngle(camera_x, camera_y, camera_z, camera_rx, camera_ry, px, py, pz, dist, &rx, &ry, &r) == true ){
				//pxAEɓĂ
				if( (fabsf(rx) < DegreeToRadian(10))&&(fabsf(ry) < DegreeToRadian(10)) ){
					*color = d3dg->GetColorCode(0.0f,1.0f,0.0f,1.0f);
					sprintf(infostr, "Weapon[%d]  %d:%d", i, lnbs, (nbs - lnbs));
					dist = sqrtf(r);
				}
			}
		}
	}

	//
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		if( SmallObjectIndex[i].GetEnableFlag() == true ){
			SmallObjectIndex[i].GetPosData(&px, &py, &pz, NULL);
			
			//_ɑΏۂ܂ł̊pxZo
			if( CheckTargetAngle(camera_x, camera_y, camera_z, camera_rx, camera_ry, px, py, pz, dist, &rx, &ry, &r) == true ){
				//pxAEɓĂ
				if( (fabsf(rx) < DegreeToRadian(10))&&(fabsf(ry) < DegreeToRadian(10)) ){
					*color = d3dg->GetColorCode(1.0f,1.0f,0.0f,1.0f);
					sprintf(infostr, "SmallObject[%d]  HP %d", i, SmallObjectIndex[i].GetHP());
					dist = sqrtf(r);
				}
			}
		}
	}

	//ݒ肳Ă邩ǂŁALȏꂽB
	if( infostr[0] == NULL ){
		return false;
	}
	return true;
}

//! @brief IuWFNg̎vZ
//! @param cmdF5id ㏸@\iF5Zjlf[^ԍi-1ŋ@\j
//! @param demomode f[h
//! @param screen ʂÂ
//! @return  0
//! @attention ʓI cmdF5id ́AF5Zgp̓vC[ԍiGetPlayerID()֐Ŏ擾jAgp -1 w肵܂B
//! @attention demomode ͎Ƀj[ʂŎgp܂BLɂƁAeeE֒e܂B
int ObjectManager::Process(int cmdF5id, bool demomode, bool screen)
{
	//̃t[̐
	for(int i=0; i<MAX_HUMAN; i++){
		Human_ontarget[i] = 0.0f;
		Human_kill[i] = 0;
		Human_headshot[i] = 0;
	}

	//lIuWFNg̏
	for(int i=0; i<MAX_HUMAN; i++){
		bool cmdF5, player;
		int rtn;
		int block_id = -1;
		int block_face = -1;
		int block_textureid = -1;
		int MoveMode;
		int JumpLandingMode;

		//vC[ǂ
		if( Player_HumanID == i ){
			player = true;
		}
		else{
			player = false;
		}

		if( i == cmdF5id ){
			cmdF5 = true;
		}
		else{
			cmdF5 = false;
		}

		rtn = HumanIndex[i].ProcessObject(CollD, BlockData, AddCollisionFlag, player, cmdF5, &block_id, &block_face);
		if( rtn == 2 ){
			//S̃GtFNg
			DeadEffect(&(HumanIndex[i]));
		}
		if( rtn == 4 ){
			int teamid, player_teamid;
			HumanIndex[i].GetParamData(NULL, NULL, NULL, &teamid);
			HumanIndex[Player_HumanID].GetParamData(NULL, NULL, NULL, &player_teamid);

			//O֌W̏
			ObjectLog->DiedHuman(-1, i, -1, teamid, player_teamid);
		}

		HumanIndex[i].SetDarkModelFlag(screen);

		//ɂubÑeNX`[ԍ擾
		if( block_id != -1 ){
			blockdata bdata;
			BlockData->Getdata(&bdata, block_id);
			block_textureid = bdata.material[block_face].textureID;
		}
		else{
			block_textureid = -1;
		}

		MoveMode = HumanIndex[i].GetMovemode(false);
		JumpLandingMode = HumanIndex[i].GetJumpLanding(false);

		//
		if( (1 <= MoveMode)&&(MoveMode <= 4) ){
			//ǉ
			//@ɂubÑeNX`[ԍA̎ޔԍƂB
			float posx, posy, posz;
			int teamid;
			int cnt;
			HumanIndex[i].GetPosData(&posx, &posy, &posz, NULL);
			HumanIndex[i].GetParamData(NULL, NULL, NULL, &teamid);
			cnt = HumanIndex[i].GetMoveMotionCount();
			GameSound->SetFootsteps(posx, posy, posz, block_textureid, MoveMode - 1, teamid, cnt);
		}

		//Wv鉹
		if( JumpLandingMode == 1 ){
			//ǉ
			float posx, posy, posz;
			int teamid;
			HumanIndex[i].GetPosData(&posx, &posy, &posz, NULL);
			HumanIndex[i].GetParamData(NULL, NULL, NULL, &teamid);
			GameSound->SetFootsteps(posx, posy, posz, block_textureid, 4, teamid, 0);
		}

		//n鉹
		if( JumpLandingMode == 2 ){
			//ǉ
			float posx, posy, posz;
			int teamid;
			HumanIndex[i].GetPosData(&posx, &posy, &posz, NULL);
			HumanIndex[i].GetParamData(NULL, NULL, NULL, &teamid);
			GameSound->SetFootsteps(posx, posy, posz, block_textureid, 5, teamid, 0);
		}
	}

	//IuWFNg̏
	for(int i=0; i<MAX_WEAPON; i++){
		WeaponIndex[i].ProcessObject(CollD);
		WeaponIndex[i].SetDarkModelFlag(screen);
	}

	//IuWFNg̏
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		SmallObjectIndex[i].ProcessObject();
		SmallObjectIndex[i].SetDarkModelFlag(screen);
	}

	if( demomode == false ){
		//eIuWFNg̏
		for(int i=0; i<MAX_BULLET; i++){
			float bx, by, bz, brx, bry;
			int speed, teamid;
			float mx, my, mz;

			CollideBullet(&BulletIndex[i]);		//蔻s
			BulletIndex[i].ProcessObject();		//vZ
			BulletIndex[i].SetDarkModelFlag(screen);

			if( BulletIndex[i].GetEnableFlag() == true ){
				//e̍WƊpx擾
				BulletIndex[i].GetParamData(NULL, NULL, &speed, &teamid, NULL, NULL);
				BulletIndex[i].GetPosData(&bx, &by, &bz, &brx, &bry);
				mx = cosf(brx)*cosf(bry)*speed;
				my = sinf(bry)*speed;
				mz = sinf(brx)*cosf(bry)*speed;
				GameSound->PassingBullet(bx, by, bz, mx, my, mz, teamid);
			}
		}
	}
	else{
		//SĂ̒eIuWFNg
		for(int i=0; i<MAX_BULLET; i++){
			if( BulletIndex[i].GetEnableFlag() == true ){
				BulletIndex[i].SetEnableFlag(false);
			}
		}
	}

	//𔭖Cۂ̃GtFNgo
	for(int i=0; i<MAX_HUMAN; i++){
		if( Human_ShotFlag[i] == true ){
			ShotWeaponEffect(i);
			Human_ShotFlag[i] = false;
		}
	}

	//GtFNgIuWFNg̏
	for(int i=0; i<MAX_EFFECT; i++){
		int id, face;
		float pos_x, pos_y, pos_z;
		if( CollideBlood(&(EffectIndex[i]), &id, &face, &pos_x, &pos_y, &pos_z) == true ){
			AddMapEffect(id, face, pos_x, pos_y, pos_z, 10.0f, DegreeToRadian(10)*GetRand(18), Resource->GetEffectBloodTexture(),  0, 0, 1.0f, -0.002f, (int)GAMEFPS * 20);
			EffectIndex[i].SetEnableFlag(false);
		}
		else{
			EffectIndex[i].ProcessObject();
			EffectIndex[i].SetDarkModelFlag(screen);
		}
	}

	if( demomode == false ){
		//֒ȅ
		for(int i=0; i<MAX_GRENADE; i++){
			float speed = GrenadeIndex[i].GetSpeed();

			//vZ
			int rcr = GrenadeIndex[i].ProcessObject(CollD);
			GrenadeIndex[i].SetDarkModelFlag(screen);

			//oEhE˕ԂȂ
			if( rcr == 1 ){
				if( speed > 3.4f ){
					//W擾AʉĐ
					float x, y, z;
					int teamid;
					GrenadeIndex[i].GetPosData(&x, &y, &z, NULL, NULL);
					GrenadeIndex[i].GetParamData(NULL, &teamid, NULL, NULL);
					GameSound->GrenadeBound(x, y, z, teamid);
				}
			}

			//Ȃ
			if( rcr == 2 ){
				GrenadeExplosion(&(GrenadeIndex[i]));
			}
		}
	}
	else{
		//SĂ̎֒e
		for(int i=0; i<MAX_GRENADE; i++){
			if( GrenadeIndex[i].GetEnableFlag() == true ){
				GrenadeIndex[i].SetEnableFlag(false);
			}
		}
	}


	//E
	for(int i=0; i<MAX_HUMAN; i++){
		if( HumanIndex[i].GetEnableFlag() == true ){
			//EŖt[ŏӖ͂Ȃ̂ŁAe2t[1񏈗ɃP`Ă܂@iOOG
			//t[ڂ̎́Af[^ԍ̕B̎́A̕B
			for(int j=(framecnt%2); j<MAX_WEAPON; j+=2){
				PickupWeapon(&HumanIndex[i], &WeaponIndex[j]);
			}
		}
	}

	for(int i=0; i<MAX_HUMAN; i++){
		if( HumanIndex[i].GetEnableFlag() == true ){
			for(int j=i+1; j<MAX_HUMAN; j++){
				if( HumanIndex[j].GetEnableFlag() == true ){

					//lm̓蔻ij
					CollideHuman(&HumanIndex[i], &HumanIndex[j]);

				}
			}
		}
	}

	//O֌W̏
	ObjectLog->Process();

	framecnt += 1;

	return 0;
}

//! @brief t[ E|G̐EwbgVbg ̎擾
//! @param id 擾l̃f[^ԍ
//! @param ontarget 󂯎|C^
//! @param kill |G̐󂯎|C^
//! @param headshot G̓ɖ󂯎|C^
//! @return Ftrue@sFfalse
bool ObjectManager::GetHumanShotInfo(int id, float *ontarget, int *kill, int *headshot)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return false; }
	if( (ontarget == NULL)||(kill == NULL)||(headshot == NULL) ){ return false; }

	*ontarget = Human_ontarget[id];
	*kill = Human_kill[id];
	*headshot = Human_headshot[id];
	return true;
}

//! @brief GtFNg\[g
//! @param camera_x JXW
//! @param camera_y JYW
//! @param camera_z JZW
//! @param data ʂ󂯎 effectdata\́@ivfFMAX_EFFECTj
//! @return LȃGtFNg̐
int ObjectManager::SortEffect(float camera_x, float camera_y, float camera_z, effectdata data[])
{
	int cnt = 0;

	//J̋߂A𐔂B
	for(int i=0; i<MAX_EFFECT; i++){
		if( EffectIndex[i].GetEnableFlag() == true ){
			float ex, ey, ez;
			float x, y, z;

			//W擾AvZAf[^ɓo^B
			EffectIndex[i].GetPosData(&ex, &ey, &ez, NULL);
			x = ex - camera_x;
			y = ey - camera_y;
			z = ez - camera_z;
			data[cnt].id = i;
			data[cnt].dist = x*x + y*y + z*z;

			//JEgZ
			cnt += 1;
		}
	}

	//P}\[g
	effectdata temp;
	for(int i=1; i<cnt; i++) {
		int j;
		temp = data[i];
		for(j=i; j>0 && data[j-1].dist<temp.dist; j--){
			data[j] = data[j-1];
		}
		data[j] = temp;
	}

	return cnt;
}

//! @brief IuWFNg̕`揈
//! @param camera_x JXW
//! @param camera_y JYW
//! @param camera_z JZW
//! @param camera_rx J̉px
//! @param camera_ry J̏cpx
//! @param HidePlayer vC[̕`ݒ@i`悷F0@`悵ȂF1@rƕ̂ݕ`悷F2j
//! @param NoModel f`斳
void ObjectManager::Render(float camera_x, float camera_y, float camera_z, float camera_rx, float camera_ry, int HidePlayer, bool NoModel)
{
	//[hW_
	d3dg->ResetWorldTransform();

	//l`
	for(int i=0; i<MAX_HUMAN; i++){
		bool DrawArm, player;

		//l̃eNX`tbVtOLȂ
		if( Human_FrameTextureRefresh[i] == true ){
			if( i > 0 ){
				//O̐lĂ镐擾
				int WeaponType = HumanIndex[i-1].GetMainWeaponTypeNO();

				if( WeaponType != ID_WEAPON_NONE ){
					int model, texture;

					//Ă΁Ã̕eNX`Kp
					Resource->GetWeaponModelTexture(WeaponType, &model, &texture);
					HumanIndex[i].SetTexture(texture);
				}
				else{
					//ԂȂAl̃eNX`̂܂ܔf
					HumanIndex[i].SetTexture( HumanIndex[i-1].GetTexture() );
				}
			}
		}

		//r̕`
		if( HidePlayer == 0 ){
			DrawArm = false;
		}
		else if( Player_HumanID != i ){	// HidePlayer != 0
			DrawArm = false;
		}
		else if( HidePlayer == 2 ){		// Player_HumanID == i
			DrawArm = true;
		}
		else{							// Player_HumanID == i && HidePlayer == 1
			continue;	//vC[g`悵ȂȂXLbv
		}

		//vC[ǂ
		if( Player_HumanID == i ){
			player = true;
		}
		else{
			player = false;
		}

		HumanIndex[i].Render(d3dg, Resource, DrawArm, player, NoModel);

		/*
		//蔻̊ȈՕ\
		float x, y, z;
		HumanIndex[i].GetPosData(&x, &y, &z, NULL);
		if( HumanIndex[i].GetHP() > 0 ){
			d3dg->SetWorldTransform(x, y, z, 0.0f, 0.0f, 1.0f);
			d3dg->Renderline(HUMAN_BULLETCOLLISION_HEAD_R, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H + HUMAN_BULLETCOLLISION_HEAD_H, 0.0f, HUMAN_BULLETCOLLISION_HEAD_R * -1, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H + HUMAN_BULLETCOLLISION_HEAD_H, 0.0f);
			d3dg->Renderline(0.0f, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H + HUMAN_BULLETCOLLISION_HEAD_H, HUMAN_BULLETCOLLISION_HEAD_R, 0.0f, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H + HUMAN_BULLETCOLLISION_HEAD_H, HUMAN_BULLETCOLLISION_HEAD_R * -1);
			d3dg->Renderline(HUMAN_BULLETCOLLISION_UP_R, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H, 0.0f, HUMAN_BULLETCOLLISION_UP_R * -1, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H, 0.0f);
			d3dg->Renderline(0.0f, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H, HUMAN_BULLETCOLLISION_UP_R, 0.0f, HUMAN_BULLETCOLLISION_LEG_H + HUMAN_BULLETCOLLISION_UP_H, HUMAN_BULLETCOLLISION_UP_R * -1);
			d3dg->Renderline(HUMAN_BULLETCOLLISION_LEG_R, HUMAN_BULLETCOLLISION_LEG_H, 0.0f, HUMAN_BULLETCOLLISION_LEG_R * -1, HUMAN_BULLETCOLLISION_LEG_H, 0.0f);
			d3dg->Renderline(0.0f, HUMAN_BULLETCOLLISION_LEG_H, HUMAN_BULLETCOLLISION_LEG_R, 0.0f, HUMAN_BULLETCOLLISION_LEG_H, HUMAN_BULLETCOLLISION_LEG_R * -1);
		}
		*/
	}

	//`
	for(int i=0; i<MAX_WEAPON; i++){
		WeaponIndex[i].Render(d3dg, NoModel);
	}

	//`
	for(int i=0; i<MAX_SMALLOBJECT; i++){
		SmallObjectIndex[i].Render(d3dg, NoModel);
	}

	//e`
	for(int i=0; i<MAX_BULLET; i++){
		BulletIndex[i].Render(d3dg, NoModel);
	}

	//֒e`
	for(int i=0; i<MAX_GRENADE; i++){
		GrenadeIndex[i].Render(d3dg, NoModel);
	}

	//GtFNg`
	d3dg->StartEffectRender();
	/*
	for(int i=0; i<MAX_EFFECT; i++){
		EffectIndex[i].Render(d3dg, camera_rx, camera_ry, NoModel);
	}
	*/
	effectdata data[MAX_EFFECT];
	int cnt = SortEffect(camera_x, camera_y, camera_z, data);
	for(int i=0; i<cnt; i++) {
		EffectIndex[ data[i].id ].Render(d3dg, camera_rx, camera_ry, NoModel);
	}
	d3dg->EndEffectRender();
}

//! @brief IuWFNgO̕`揈
//! @param x  xW
//! @param y  yW
void ObjectManager::RenderLog(int x, int y)
{
	ObjectLog->Render(x, y);
}

//! @brief f[^̉
void ObjectManager::Cleanup()
{
	//|Cgf[^
	CleanupPointDataToObject();
}

//! @brief RXgN^
BulletObjectHumanIndex::BulletObjectHumanIndex()
{
	HumanIndex = new bool[MAX_HUMAN];

	Init();
}

//! @brief fBXgN^
BulletObjectHumanIndex::~BulletObjectHumanIndex()
{
	if( HumanIndex != NULL ){ delete [] HumanIndex; }
}

//! @brief 
void BulletObjectHumanIndex::Init()
{
	for(int i=0; i<MAX_HUMAN; i++){
		HumanIndex[i] = false;
	}
}

//! @brief tO擾
//! @param id l̃f[^ԍ
//! @return tOl
bool BulletObjectHumanIndex::GetIndexFlag(int id)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return false; }
	return HumanIndex[id];
}

//! @brief tOL
//! @param id l̃f[^ԍ
void BulletObjectHumanIndex::SetIndexFlag(int id)
{
	if( (id < 0)||(MAX_HUMAN <= id) ){ return; }
	HumanIndex[id] = true;
}

//! @brief RXgN^
ObjectManagerLog::ObjectManagerLog()
{
	d3dg = NULL;
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN; i++){
		TextStr[i] = new char [MAX_OBJECTMANAGER_LOGLINES];
	}
	ClearLog();
}

//! @brief fBXgN^
ObjectManagerLog::~ObjectManagerLog()
{
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN; i++){
		delete [] TextStr[i];
	}
}

//! @brief QƂNXݒ
//! @param in_d3dg `揈NX
//! @attention ̊֐ŐݒsȂƁANX̂@\܂B
void ObjectManagerLog::SetClass(D3DGraphics *in_d3dg)
{
	d3dg = in_d3dg;
}

//! @brief IuWFNgO
void ObjectManagerLog::ClearLog()
{
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN; i++){
		TextCnt[i] = -1;
		TextStr[i][0] = '\0';
		TextColor[i] = 0;
	}
}

//! @brief ǋL
//! @param str @isR[hF<b>s</b>j
void ObjectManagerLog::InfoLog(const char *str)
{
	if( str == NULL ){ return; }
	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, d3dg->GetColorCode(0.0f,1.0f,0.0f,1.0f));
}

//! @brief lǉ
//! @param humanID lID
//! @param TeamID `[ԍ
//! @param PlayerTeamID vC[̃`[ԍ
void ObjectManagerLog::AddHuman(int humanID, int TeamID, int PlayerTeamID)
{
	char str[64];
	int str_color;

	//̐F
	if( PlayerTeamID == TeamID ){ str_color = d3dg->GetColorCode(0.0f,1.0f,1.0f,1.0f); }
	else{ str_color = d3dg->GetColorCode(1.0f,0.5f,0.0f,1.0f); }

	sprintf(str, "Added human[%d]", humanID);

	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, str_color);
}

//! @brief lS
//! @param ShothumanID  UlID
//! @param HitHumanID U󂯂lID
//! @param ShothumanTeamID Ul̃`[ԍ
//! @param HitHumanTeamID U󂯂l̃`[ԍ
//! @param PlayerTeamID vC[̃`[ԍ
void ObjectManagerLog::DiedHuman(int ShothumanID, int HitHumanID, int ShothumanTeamID, int HitHumanTeamID, int PlayerTeamID)
{
	char str[64];
	int str_color;

	//̐F
	if( PlayerTeamID == HitHumanTeamID ){ str_color = d3dg->GetColorCode(1.0f,0.5f,0.0f,1.0f); }
	else{ str_color = d3dg->GetColorCode(0.0f,1.0f,1.0f,1.0f); }

	if( (ShothumanID == -1)||(ShothumanID == HitHumanID) ){
		sprintf(str, "human[%d] died", HitHumanID);
	}
	else{
		sprintf(str, "human[%d] killed human[%d]", ShothumanID, HitHumanID);

		//TK
		if( ShothumanTeamID == HitHumanTeamID ){
			strcat(str, " (TK)");
		}
	}

	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, str_color);
}

//! @brief lh
//! @param humanID lID
//! @param TeamID `[ԍ
//! @param PlayerTeamID vC[̃`[ԍ
void ObjectManagerLog::ReviveHuman(int humanID, int TeamID, int PlayerTeamID)
{
	char str[64];
	int str_color;

	//̐F
	if( PlayerTeamID == TeamID ){ str_color = d3dg->GetColorCode(0.0f,1.0f,1.0f,1.0f); }
	else{ str_color = d3dg->GetColorCode(1.0f,0.5f,0.0f,1.0f); }

	sprintf(str, "Revived human[%d]", humanID);

	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, str_color);
}

//! @brief ǉ
//! @param objID ID
void ObjectManagerLog::AddSmallObject(int objID)
{
	char str[64];
	sprintf(str, "Added SmallObject[%d]", objID);
	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, d3dg->GetColorCode(1.0f,1.0f,0.0f,1.0f));
}

//! @brief j󂵂
//! @param objID ID
void ObjectManagerLog::BreakSmallObject(int objID)
{
	char str[64];
	sprintf(str, "Breaked SmallObject[%d]", objID);
	AddTextLog(MAX_OBJECTMANAGER_LOGCNT, str, d3dg->GetColorCode(1.0f,1.0f,0.0f,1.0f));
}

//! @brief IuWFNgOǉ
//! @param cnt `t[
//! @param str @isR[hF<b>s</b>j
//! @param color F
//! @return 1s㏑Ftrue@ǉ̂݁Ffalse
bool ObjectManagerLog::AddTextLog(int cnt, const char *str, int color)
{
	if( str == NULL ){ return false; }

	//󂢂ĂsȂA̍sɏďI
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN; i++){
		if( TextCnt[i] == -1 ){
			TextCnt[i] = cnt;
			strcpy(TextStr[i], str);
			TextColor[i] = color;
			return false;
		}
	}

	//󂢂ĂsȂ΁A1sÂl߂
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN-1; i++){
		TextCnt[i] = TextCnt[i+1];
		strcpy(TextStr[i], TextStr[i+1]);
		TextColor[i] = TextColor[i+1];
	}

	//Ō̍sɏ
	TextCnt[MAX_OBJECTMANAGER_LOGLEN-1] = cnt;
	strcpy(TextStr[MAX_OBJECTMANAGER_LOGLEN-1], str);
	TextColor[MAX_OBJECTMANAGER_LOGLEN-1] = color;
	return true;
}

//! @brief IuWFNgO̎vZ
void ObjectManagerLog::Process()
{
	//esƂɃJEg
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN; i++){
		if( TextCnt[i] == 0 ){
			TextCnt[i] = -1;
			TextStr[i][0] = '\0';
			TextColor[i] = 0;
		}
		else if( TextCnt[i] > 0 ){
			TextCnt[i] -= 1;
		}
	}

	//[JEg̍sl߂
	for(int i=0; i<MAX_OBJECTMANAGER_LOGLEN-1; i++){
		if( TextCnt[i] == 0 ){
			for(int j=i; j<MAX_OBJECTMANAGER_LOGLEN-1; j++){
				TextCnt[j] = TextCnt[j+1];
				strcpy(TextStr[j], TextStr[j+1]);
				TextColor[j] = TextColor[j+1];
			}

			//Ō̍s͋s
			TextCnt[MAX_OBJECTMANAGER_LOGLEN-1] = -1;
			TextStr[MAX_OBJECTMANAGER_LOGLEN-1][0] = '\0';
			TextColor[MAX_OBJECTMANAGER_LOGLEN-1] = 0;
		}
	}
}

//! @brief IuWFNgO̕`揈
//! @param x  xW
//! @param y  yW
//! @attention ォtŕ`悳܂B
void ObjectManagerLog::Render(int x, int y)
{
#ifdef ENABLE_DEBUGCONSOLE
	if( d3dg == NULL ){ return; }

	for(int i=MAX_OBJECTMANAGER_LOGLEN-1; i>=0; i--){
		if( TextCnt[i] >= 0 ){
			d3dg->Draw2DTextureDebugFontText(x+2+1, y+2+1, TextStr[i], d3dg->GetColorCode(0.1f,0.1f,0.1f,1.0f));
			d3dg->Draw2DTextureDebugFontText(x+2, y+2, TextStr[i], TextColor[i]);
		}
		y += 20;
	}
#endif
}